@@ -88,32 +88,50 @@ module Sequel
   module Postgres
     CONVERTED_EXCEPTIONS << PGError
     
-    # Hash with integer keys and proc values for converting PostgreSQL types.
-    PG_TYPES = {}
+    TYPE_TRANSLATOR = tt = Class.new do
+      def boolean(s) s == 't' end
+      def bytea(s) ::Sequel::SQL::Blob.new(Adapter.unescape_bytea(s)) end
+      def integer(s) s.to_i end
+      def float(s) s.to_f end
+      def numeric(s) ::BigDecimal.new(s) end
+      def date(s) ::Sequel.string_to_date(s) end
+      def date_iso(s) ::Date.new(*s.split("-").map{|x| x.to_i}) end
+      def time(s) ::Sequel.string_to_time(s) end
+      def timestamp(s) ::Sequel.database_to_application_timestamp(s) end
+    end.new
 
-    # Use a single proc for each type to conserve memory
-    PG_TYPE_PROCS  = {
-      [16] =>  lambda{|s| s == 't'}, # boolean
-      [17] => lambda{|s| ::Sequel::SQL::Blob.new(Adapter.unescape_bytea(s))}, # bytea
-      [20, 21, 22, 23, 26] => lambda{|s| s.to_i}, # integer
-      [700, 701] => lambda{|s| s.to_f}, # float
-      [790, 1700] => lambda{|s| BigDecimal.new(s)}, # numeric
-      [1082] => lambda{|s| @use_iso_date_format ? Date.new(*s.split("-").map{|x| x.to_i}) : Sequel.string_to_date(s)}, # date
-      [1083, 1266] => lambda{|s| Sequel.string_to_time(s)}, # time
-      [1114, 1184] => lambda{|s| Sequel.database_to_application_timestamp(s)}, # timestamp
-    }
-    PG_TYPE_PROCS.each do |k,v|
+    # Hash with type name symbols and callable values for converting PostgreSQL types.
+    # Non-builtin types that don't have fixed numbers should use this to register
+    # conversion procs.
+    PG_NAMED_TYPES = {}
+
+    # Hash with integer keys and callable values for converting PostgreSQL types.
+    PG_TYPES = {}
+    {
+      [16] => tt.method(:boolean),
+      [17] => tt.method(:bytea),
+      [20, 21, 22, 23, 26] => tt.method(:integer),
+      [700, 701] => tt.method(:float),
+      [790, 1700] => tt.method(:numeric),
+      [1083, 1266] => tt.method(:time),
+      [1114, 1184] => tt.method(:timestamp)
+    }.each do |k,v|
       k.each{|n| PG_TYPES[n] = v}
     end
     
-    @use_iso_date_format = true
-
     class << self
       # As an optimization, Sequel sets the date style to ISO, so that PostgreSQL provides
       # the date in a known format that Sequel can parse faster.  This can be turned off
       # if you require a date style other than ISO.
-      attr_accessor :use_iso_date_format
+      attr_reader :use_iso_date_format
     end
+
+    # Modify the type translator for the date type depending on the value given.
+    def self.use_iso_date_format=(v)
+      PG_TYPES[1082] = TYPE_TRANSLATOR.method(v ? :date_iso : :date)
+      @use_iso_date_format = v
+    end
+    self.use_iso_date_format = true
     
     # PGconn subclass for connection specific methods used with the
     # pg, postgres, or postgres-pr driver.
@@ -133,29 +151,33 @@ module Sequel
         super
         if Postgres.use_iso_date_format
           sql = "SET DateStyle = 'ISO'"
-          @db.log_info(sql)
           execute(sql)
         end
         @prepared_statements = {} if SEQUEL_POSTGRES_USES_PG
       end
-
-      # Execute the given SQL with this connection.  If a block is given,
-      # yield the results, otherwise, return the number of changed rows.
-      def execute(sql, args=nil)
-        q = nil
+      
+      # Raise a Sequel::DatabaseDisconnectError if a PGError is raised and
+      # the connection status cannot be determined or it is not OK.
+      def check_disconnect_errors
         begin
-          q = args ? async_exec(sql, args) : async_exec(sql)
-        rescue PGError => e
+          yield
+        rescue PGError =>e
           begin
             s = status
           rescue PGError
             raise Sequel.convert_exception_class(e, Sequel::DatabaseDisconnectError)
           end
           status_ok = (s == Adapter::CONNECTION_OK)
-          status_ok ? raise : Sequel.convert_exception_class(e, Sequel::DatabaseDisconnectError)
+          status_ok ? raise : raise(Sequel.convert_exception_class(e, Sequel::DatabaseDisconnectError))
         ensure
           block if status_ok
         end
+      end
+
+      # Execute the given SQL with this connection.  If a block is given,
+      # yield the results, otherwise, return the number of changed rows.
+      def execute(sql, args=nil)
+        q = check_disconnect_errors{@db.log_yield(sql, args){args ? async_exec(sql, args) : async_exec(sql)}}
         begin
           block_given? ? yield(q) : q.cmd_tuples
         ensure
@@ -177,6 +199,10 @@ module Sequel
       include Sequel::Postgres::DatabaseMethods
       
       set_adapter_scheme :postgres
+
+      # A hash of conversion procs, keyed by type integer (oid) and
+      # having callable values for the conversion proc for that type.
+      attr_reader :conversion_procs
       
       # Add the primary_keys and primary_key_sequences instance variables,
       # so we can get the correct return values for inserted rows.
@@ -208,6 +234,7 @@ module Sequel
         end
         conn.db = self
         conn.apply_connection_settings
+        @conversion_procs ||= get_conversion_procs(conn)
         conn
       end
       
@@ -218,13 +245,9 @@ module Sequel
       
       # Execute the given SQL with the given args on an available connection.
       def execute(sql, opts={}, &block)
-        return execute_prepared_statement(sql, opts, &block) if Symbol === sql
-        begin
-          log_info(sql, opts[:arguments])
+        check_database_errors do
+          return execute_prepared_statement(sql, opts, &block) if Symbol === sql
           synchronize(opts[:server]){|conn| conn.execute(sql, opts[:arguments], &block)}
-        rescue => e
-          log_info(e.message)
-          raise_error(e, :classes=>CONVERTED_EXCEPTIONS)
         end
       end
       
@@ -232,25 +255,25 @@ module Sequel
       # automatically generated).
       def execute_insert(sql, opts={})
         return execute(sql, opts) if Symbol === sql
-        begin 
-          log_info(sql, opts[:arguments])
+        check_database_errors do
           synchronize(opts[:server]) do |conn|
             conn.execute(sql, opts[:arguments])
             insert_result(conn, opts[:table], opts[:values])
           end
-        rescue => e
-          log_info(e.message)
-          raise_error(e, :classes=>CONVERTED_EXCEPTIONS)
         end
       end
       
       private
-
-      # PostgreSQL doesn't need the connection pool to convert exceptions.
-      def connection_pool_default_options
-        super.merge(:pool_convert_exceptions=>false)
-      end
       
+      # Convert exceptions raised from the block into DatabaseErrors.
+      def check_database_errors
+        begin
+          yield
+        rescue => e
+          raise_error(e, :classes=>CONVERTED_EXCEPTIONS)
+        end
+      end
+
       # Disconnect given connection
       def disconnect_connection(conn)
         begin
@@ -275,16 +298,12 @@ module Sequel
         synchronize(opts[:server]) do |conn|
           unless conn.prepared_statements[ps_name] == sql
             if conn.prepared_statements.include?(ps_name)
-              s = "DEALLOCATE #{ps_name}"
-              log_info(s)
-              conn.execute(s) unless conn.prepared_statements[ps_name] == sql
+              conn.execute("DEALLOCATE #{ps_name}") unless conn.prepared_statements[ps_name] == sql
             end
             conn.prepared_statements[ps_name] = sql
-            log_info("PREPARE #{ps_name} AS #{sql}")
-            conn.prepare(ps_name, sql)
+            conn.check_disconnect_errors{log_yield("PREPARE #{ps_name} AS #{sql}"){conn.prepare(ps_name, sql)}}
           end
-          log_info("EXECUTE #{ps_name}", args)
-          q = conn.exec_prepared(ps_name, args)
+          q = conn.check_disconnect_errors{log_yield("EXECUTE #{ps_name}", args){conn.exec_prepared(ps_name, args)}}
           if opts[:table] && opts[:values]
             insert_result(conn, opts[:table], opts[:values])
           else
@@ -296,6 +315,19 @@ module Sequel
           end
         end
       end
+
+      # Return the conversion procs hash to use for this database
+      def get_conversion_procs(conn)
+        procs = PG_TYPES.dup
+        conn.execute("SELECT oid, typname FROM pg_type where typtype = 'b'") do |res|
+          res.ntuples.times do |recnum|
+            if pr = PG_NAMED_TYPES[res.getvalue(recnum, 1).to_sym]
+              procs[res.getvalue(recnum, 0).to_i] ||= pr
+            end
+          end
+        end
+        procs
+      end
     end
     
     # Dataset class for PostgreSQL datasets that use the pg, postgres, or
@@ -363,6 +395,11 @@ module Sequel
             end
             LiteralString.new("#{prepared_arg_placeholder}#{i}#{"::#{type}" if type}")
           end
+
+          # Always assume a prepared argument.
+          def prepared_arg?(k)
+           true
+          end
         end
 
         # Allow use of bind arguments for PostgreSQL using the pg driver.
@@ -474,8 +511,9 @@ module Sequel
       # field numers, type conversion procs, and name symbol arrays.
       def fetch_rows_set_cols(res)
         cols = []
+        procs = db.conversion_procs
         res.nfields.times do |fieldnum|
-          cols << [fieldnum, PG_TYPES[res.ftype(fieldnum)], output_identifier(res.fname(fieldnum))]
+          cols << [fieldnum, procs[res.ftype(fieldnum)], output_identifier(res.fname(fieldnum))]
         end
         @columns = cols.map{|c| c.at(2)}
         cols
@@ -506,3 +544,16 @@ module Sequel
     end
   end
 end
+
+if SEQUEL_POSTGRES_USES_PG
+  begin
+    require 'sequel_pg'
+  rescue LoadError
+    if RUBY_PLATFORM =~ /mingw|mswin/
+      begin
+        require "#{RUBY_VERSION[0...3]}/sequel_pg"
+      rescue LoadError
+      end
+    end
+  end
+end
