@@ -3,9 +3,26 @@ module Sequel
   Dataset::NON_SQL_OPTIONS << :on_duplicate_key_update
 
   module MySQL
+    @convert_tinyint_to_bool = true
+
     class << self
-      # Set the default options used for CREATE TABLE
-      attr_accessor :default_charset, :default_collate, :default_engine
+      # Sequel converts the column type tinyint(1) to a boolean by default when
+      # using the native MySQL or Mysql2 adapter.  You can turn off the conversion by setting
+      # this to false. This setting is ignored when connecting to MySQL via the do or jdbc
+      # adapters, both of which automatically do the conversion.
+      attr_accessor :convert_tinyint_to_bool
+
+      # Set the default charset used for CREATE TABLE.  You can pass the
+      # :charset option to create_table to override this setting.
+      attr_accessor :default_charset
+
+      # Set the default collation used for CREATE TABLE.  You can pass the
+      # :collate option to create_table to override this setting.
+      attr_accessor :default_collate
+
+      # Set the default engine used for CREATE TABLE.  You can pass the
+      # :engine option to create_table to override this setting.
+      attr_accessor :default_engine
     end
 
     # Methods shared by Database instances that connect to MySQL,
@@ -13,6 +30,7 @@ module Sequel
     module DatabaseMethods
       AUTO_INCREMENT = 'AUTO_INCREMENT'.freeze
       CAST_TYPES = {String=>:CHAR, Integer=>:SIGNED, Time=>:DATETIME, DateTime=>:DATETIME, Numeric=>:DECIMAL, BigDecimal=>:DECIMAL, File=>:BINARY}
+      COLUMN_DEFINITION_ORDER = [:null, :default, :unique, :primary_key, :auto_increment, :references]
       PRIMARY = 'PRIMARY'.freeze
       
       # MySQL's cast rules are restrictive in that you can't just cast to any possible
@@ -21,18 +39,23 @@ module Sequel
         CAST_TYPES[type] || super
       end
 
+      # Commit an existing prepared transaction with the given transaction
+      # identifier string.
+      def commit_prepared_transaction(transaction_id)
+        run("XA COMMIT #{literal(transaction_id)}")
+      end
+
       # MySQL uses the :mysql database type
       def database_type
         :mysql
       end
 
-      # Return a hash containing index information. Hash keys are index name symbols.
-      # Values are subhashes with two keys, :columns and :unique.  The value of :columns
-      # is an array of symbols of column names.  The value of :unique is true or false
-      # depending on if the index is unique.
+      # Use SHOW INDEX FROM to get the index information for the
+      # table.
       #
-      # Does not include the primary key index or indexes on partial keys.
-      def indexes(table)
+      # By default partial indexes are not included, you can use the
+      # option :partial to override this.
+      def indexes(table, opts={})
         indexes = {}
         remove_indexes = []
         m = output_identifier_meth
@@ -41,33 +64,53 @@ module Sequel
           name = r[:Key_name]
           next if name == PRIMARY
           name = m.call(name)
-          remove_indexes << name if r[:Sub_part]
+          remove_indexes << name if r[:Sub_part] && ! opts[:partial]
           i = indexes[name] ||= {:columns=>[], :unique=>r[:Non_unique] != 1}
           i[:columns] << m.call(r[:Column_name])
         end
         indexes.reject{|k,v| remove_indexes.include?(k)}
       end
 
+      # Rollback an existing prepared transaction with the given transaction
+      # identifier string.
+      def rollback_prepared_transaction(transaction_id)
+        run("XA ROLLBACK #{literal(transaction_id)}")
+      end
+
       # Get version of MySQL server, used for determined capabilities.
       def server_version
         m = /(\d+)\.(\d+)\.(\d+)/.match(get(SQL::Function.new(:version)))
         @server_version ||= (m[1].to_i * 10000) + (m[2].to_i * 100) + m[3].to_i
       end
       
-      # Return an array of symbols specifying table names in the current database.
-      #
-      # Options:
-      # * :server - Set the server to use
-      def tables(opts={})
-        m = output_identifier_meth
-        metadata_dataset.with_sql('SHOW TABLES').server(opts[:server]).map{|r| m.call(r.values.first)}
+      # MySQL supports CREATE TABLE IF NOT EXISTS syntax.
+      def supports_create_table_if_not_exists?
+        true
       end
       
+      # MySQL supports prepared transactions (two-phase commit) using XA
+      def supports_prepared_transactions?
+        true
+      end
+
       # MySQL supports savepoints
       def supports_savepoints?
         true
       end
 
+      # MySQL supports transaction isolation levels
+      def supports_transaction_isolation_levels?
+        true
+      end
+
+      # Return an array of symbols specifying table names in the current database.
+      #
+      # Options:
+      # * :server - Set the server to use
+      def tables(opts={})
+        full_tables('BASE TABLE', opts)
+      end
+      
       # Changes the database in use by issuing a USE statement.  I would be
       # very careful if I used this.
       def use(db_name)
@@ -77,6 +120,14 @@ module Sequel
         self
       end
       
+      # Return an array of symbols specifying view names in the current database.
+      #
+      # Options:
+      # * :server - Set the server to use
+      def views(opts={})
+        full_tables('VIEW', opts)
+      end
+      
       private
       
       # Use MySQL specific syntax for rename column, set column type, and
@@ -113,6 +164,28 @@ module Sequel
         AUTO_INCREMENT
       end
       
+      # MySQL needs to set transaction isolation before begining a transaction
+      def begin_new_transaction(conn, opts)
+        set_transaction_isolation(conn, opts)
+        log_connection_execute(conn, begin_transaction_sql)
+      end
+
+      # Use XA START to start a new prepared transaction if the :prepare
+      # option is given.
+      def begin_transaction(conn, opts={})
+        if s = opts[:prepare]
+          log_connection_execute(conn, "XA START #{literal(s)}")
+          conn
+        else
+          super
+        end
+      end
+
+      # The order of the column definition, as an array of symbols.
+      def column_definition_order
+        COLUMN_DEFINITION_ORDER
+      end
+
       # MySQL doesn't allow default values on text columns, so ignore if it the
       # generic text type is used
       def column_definition_sql(column)
@@ -120,19 +193,36 @@ module Sequel
         super
       end
       
-      # MySQL doesn't handle references as column constraints, it must use a separate table constraint
-      def column_references_column_constraint_sql(column)
-        "#{", FOREIGN KEY (#{quote_identifier(column[:name])})" unless column[:type] == :check}#{column_references_sql(column)}"
+      # Prepare the XA transaction for a two-phase commit if the
+      # :prepare option is given.
+      def commit_transaction(conn, opts={})
+        if s = opts[:prepare]
+          log_connection_execute(conn, "XA END #{literal(s)}")
+          log_connection_execute(conn, "XA PREPARE #{literal(s)}")
+        else
+          super
+        end
       end
-      
+
       # Use MySQL specific syntax for engine type and character encoding
       def create_table_sql(name, generator, options = {})
-        engine = options.include?(:engine) ? options[:engine] : Sequel::MySQL.default_engine
-        charset = options.include?(:charset) ? options[:charset] : Sequel::MySQL.default_charset
-        collate = options.include?(:collate) ? options[:collate] : Sequel::MySQL.default_collate
+        engine = options.fetch(:engine, Sequel::MySQL.default_engine)
+        charset = options.fetch(:charset, Sequel::MySQL.default_charset)
+        collate = options.fetch(:collate, Sequel::MySQL.default_collate)
+        generator.columns.each do |c|
+          if t = c.delete(:table)
+            generator.foreign_key([c[:name]], t, c.merge(:name=>nil, :type=>:foreign_key))
+          end
+        end
         "#{super}#{" ENGINE=#{engine}" if engine}#{" DEFAULT CHARSET=#{charset}" if charset}#{" DEFAULT COLLATE=#{collate}" if collate}"
       end
 
+      # Backbone of the tables and views support using SHOW FULL TABLES.
+      def full_tables(type, opts)
+        m = output_identifier_meth
+        metadata_dataset.with_sql('SHOW FULL TABLES').server(opts[:server]).map{|r| m.call(r.values.first) if r.delete(:Table_type) == type}.compact
+      end
+
       # MySQL folds unquoted identifiers to lowercase, so it shouldn't need to upcase identifiers on input.
       def identifier_input_method_default
         nil
@@ -158,6 +248,17 @@ module Sequel
         "CREATE #{index_type}INDEX #{index_name}#{using} ON #{quote_schema_table(table_name)} #{literal(index[:columns])}"
       end
       
+      # Rollback the currently open XA transaction
+      def rollback_transaction(conn, opts={})
+        if s = opts[:prepare]
+          log_connection_execute(conn, "XA END #{literal(s)}")
+          log_connection_execute(conn, "XA PREPARE #{literal(s)}")
+          log_connection_execute(conn, "XA ROLLBACK #{literal(s)}")
+        else
+          super
+        end
+      end
+
       # MySQL treats integer primary keys as autoincrementing.
       def schema_autoincrementing_primary_key?(schema)
         super and schema[:db_type] =~ /int/io
@@ -168,7 +269,7 @@ module Sequel
         m = output_identifier_meth
         im = input_identifier_meth
         metadata_dataset.with_sql("DESCRIBE ?", SQL::Identifier.new(im.call(table_name))).map do |row|
-          row.delete(:Extra)
+          row[:auto_increment] = true if row.delete(:Extra).to_s =~ /auto_increment/io
           row[:allow_null] = row.delete(:Null) == 'YES'
           row[:default] = row.delete(:Default)
           row[:primary_key] = row.delete(:Key) == 'PRI'
@@ -179,6 +280,22 @@ module Sequel
         end
       end
 
+      # Respect the :size option if given to produce
+      # tinyblob, mediumblob, and longblob if :tiny,
+      # :medium, or :long is given.
+      def type_literal_generic_file(column)
+        case column[:size]
+        when :tiny    # < 2^8 bytes
+          :tinyblob
+        when :medium  # < 2^24 bytes
+          :mediumblob
+        when :long    # < 2^32 bytes
+          :longblob
+        else          # 2^16 bytes
+          :blob
+        end
+      end
+
       # MySQL has both datetime and timestamp classes, most people are going
       # to want datetime
       def type_literal_generic_datetime(column)
@@ -202,9 +319,11 @@ module Sequel
       BOOL_TRUE = '1'.freeze
       BOOL_FALSE = '0'.freeze
       COMMA_SEPARATOR = ', '.freeze
+      FOR_SHARE = ' LOCK IN SHARE MODE'.freeze
+      SQL_CALC_FOUND_ROWS = ' SQL_CALC_FOUND_ROWS'.freeze
       DELETE_CLAUSE_METHODS = Dataset.clause_methods(:delete, %w'from where order limit')
       INSERT_CLAUSE_METHODS = Dataset.clause_methods(:insert, %w'ignore into columns values on_duplicate_key_update')
-      SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'distinct columns from join where group having compounds order limit')
+      SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'distinct calc_found_rows columns from join where group having compounds order limit lock')
       UPDATE_CLAUSE_METHODS = Dataset.clause_methods(:update, %w'table set where order limit')
       
       # MySQL specific syntax for LIKE/REGEXP searches, as well as
@@ -219,10 +338,30 @@ module Sequel
           else
             literal(args.at(0))
           end
+        when :'B~'
+          "CAST(~#{literal(args.at(0))} AS SIGNED INTEGER)"
         else
           super(op, args)
         end
       end
+      
+      # Use GROUP BY instead of DISTINCT ON if arguments are provided.
+      def distinct(*args)
+        args.empty? ? super : group(*args)
+      end
+
+      # Sets up the select methods to use SQL_CALC_FOUND_ROWS option.
+      #
+      #   dataset.calc_found_rows.limit(10)
+      #   # SELECT SQL_CALC_FOUND_ROWS * FROM table LIMIT 10
+      def calc_found_rows
+        clone(:calc_found_rows => true)
+      end
+      
+      # Return a cloned dataset which will use LOCK IN SHARE MODE to lock returned rows.
+      def for_share
+        lock_style(:share)
+      end
 
       # Adds full text filter
       def full_text_search(cols, terms, opts = {})
@@ -241,10 +380,10 @@ module Sequel
       
       # Transforms an CROSS JOIN to an INNER JOIN if the expr is not nil.
       # Raises an error on use of :full_outer type, since MySQL doesn't support it.
-      def join_table(type, table, expr=nil, table_alias={})
+      def join_table(type, table, expr=nil, table_alias={}, &block)
         type = :inner if (type == :cross) && !expr.nil?
         raise(Sequel::Error, "MySQL doesn't support FULL OUTER JOIN") if type == :full_outer
-        super(type, table, expr, table_alias)
+        super(type, table, expr, table_alias, &block)
       end
       
       # Transforms :natural_inner to NATURAL LEFT JOIN and straight to
@@ -257,23 +396,19 @@ module Sequel
         end
       end
       
-      # Sets up multi_insert or import to use INSERT IGNORE.
+      # Sets up the insert methods to use INSERT IGNORE.
       # Useful if you have a unique key and want to just skip
       # inserting rows that violate the unique key restriction.
       #
-      # Example:
-      #
-      # dataset.insert_ignore.multi_insert(
-      #  [{:name => 'a', :value => 1}, {:name => 'b', :value => 2}]
-      # )
-      #
-      # INSERT IGNORE INTO tablename (name, value) VALUES (a, 1), (b, 2)
-      #
+      #   dataset.insert_ignore.multi_insert(
+      #    [{:name => 'a', :value => 1}, {:name => 'b', :value => 2}]
+      #   )
+      #   # INSERT IGNORE INTO tablename (name, value) VALUES (a, 1), (b, 2)
       def insert_ignore
         clone(:insert_ignore=>true)
       end
       
-      # Sets up multi_insert or import to use ON DUPLICATE KEY UPDATE
+      # Sets up the insert methods to use ON DUPLICATE KEY UPDATE
       # If you pass no arguments, ALL fields will be
       # updated with the new values.  If you pass the fields you
       # want then ONLY those field will be updated.
@@ -281,22 +416,17 @@ module Sequel
       # Useful if you have a unique key and want to update
       # inserting rows that violate the unique key restriction.
       #
-      # Examples:
-      #
-      # dataset.on_duplicate_key_update.multi_insert(
-      #  [{:name => 'a', :value => 1}, {:name => 'b', :value => 2}]
-      # )
-      #
-      # INSERT INTO tablename (name, value) VALUES (a, 1), (b, 2)
-      # ON DUPLICATE KEY UPDATE name=VALUES(name), value=VALUES(value)
-      #
-      # dataset.on_duplicate_key_update(:value).multi_insert(
-      #  [{:name => 'a', :value => 1}, {:name => 'b', :value => 2}]
-      # )
-      #
-      # INSERT INTO tablename (name, value) VALUES (a, 1), (b, 2)
-      # ON DUPLICATE KEY UPDATE value=VALUES(value)
+      #   dataset.on_duplicate_key_update.multi_insert(
+      #    [{:name => 'a', :value => 1}, {:name => 'b', :value => 2}]
+      #   )
+      #   # INSERT INTO tablename (name, value) VALUES (a, 1), (b, 2)
+      #   # ON DUPLICATE KEY UPDATE name=VALUES(name), value=VALUES(value)
       #
+      #   dataset.on_duplicate_key_update(:value).multi_insert(
+      #     [{:name => 'a', :value => 1}, {:name => 'b', :value => 2}]
+      #   )
+      #   # INSERT INTO tablename (name, value) VALUES (a, 1), (b, 2)
+      #   # ON DUPLICATE KEY UPDATE value=VALUES(value)
       def on_duplicate_key_update(*args)
         clone(:on_duplicate_key_update => args)
       end
@@ -306,6 +436,12 @@ module Sequel
         [insert_sql(columns, LiteralString.new('VALUES ' + values.map {|r| literal(Array(r))}.join(COMMA_SEPARATOR)))]
       end
       
+      # MySQL uses the number of rows actually modified in the update,
+      # instead of the number of matched by the filter.
+      def provides_accurate_rows_matched?
+        false
+      end
+      
       # MySQL uses the nonstandard ` (backtick) for quoting identifiers.
       def quoted_identifier(c)
         "`#{c}`"
@@ -317,9 +453,10 @@ module Sequel
         clone(:replace=>true).insert_sql(*values)
       end
       
-      #  does not support DISTINCT ON
+      # MySQL can emulate DISTINCT ON with its non-standard GROUP BY implementation,
+      # though the rows returned cannot be made deterministic through ordering.
       def supports_distinct_on?
-        false
+        true
       end
 
       # MySQL does not support INTERSECT or EXCEPT
@@ -327,6 +464,11 @@ module Sequel
         false
       end
       
+      # MySQL supports modifying joined datasets
+      def supports_modifying_joins?
+        true
+      end
+
       # MySQL does support fractional timestamps in literal timestamps, but it
       # ignores them.  Also, using them seems to cause problems on 1.9.  Since
       # they are ignored anyway, not using them is probably best.
@@ -347,6 +489,17 @@ module Sequel
       def delete_clause_methods
         DELETE_CLAUSE_METHODS
       end
+      
+      # Consider the first table in the joined dataset is the table to delete
+      # from, but include the others for the purposes of selecting rows.
+      def delete_from_sql(sql)
+        if joined_dataset?
+          sql << " #{source_list(@opts[:from][0..0])} FROM #{source_list(@opts[:from])}"
+          select_join_sql(sql)
+        else
+          super
+        end
+      end
 
       # MySQL supports the IGNORE and ON DUPLICATE KEY UPDATE clauses for INSERT statements
       def insert_clause_methods
@@ -424,6 +577,16 @@ module Sequel
       def select_clause_methods
         SELECT_CLAUSE_METHODS
       end
+      
+      # Support FOR SHARE locking when using the :share lock style.
+      def select_lock_sql(sql)
+        @opts[:lock] == :share ? (sql << FOR_SHARE) : super
+      end
+
+      # MySQL specific SQL_CALC_FOUND_ROWS option
+      def select_calc_found_rows_sql(sql)
+        sql << SQL_CALC_FOUND_ROWS if opts[:calc_found_rows]
+      end
 
       # MySQL supports the ORDER BY and LIMIT clauses for UPDATE statements
       def update_clause_methods
