@@ -1,11 +1,15 @@
 module Sequel
   module SQLite
+    # No matter how you connect to SQLite, the following Database options
+    # can be used to set PRAGMAs on connections in a thread-safe manner:
+    # :auto_vacuum, :foreign_keys, :synchronous, and :temp_store.
     module DatabaseMethods
       AUTO_VACUUM = [:none, :full, :incremental].freeze
       PRIMARY_KEY_INDEX_RE = /\Asqlite_autoindex_/.freeze
       SYNCHRONOUS = [:off, :normal, :full].freeze
-      TABLES_FILTER = "type = 'table' AND NOT name = 'sqlite_sequence'"
+      TABLES_FILTER = "type = 'table' AND NOT name = 'sqlite_sequence'".freeze
       TEMP_STORE = [:default, :file, :memory].freeze
+      VIEWS_FILTER = "type = 'view'".freeze
       
       # Run all alter_table commands in a transaction.  This is technically only
       # needed for drop column.
@@ -21,7 +25,8 @@ module Sequel
       end
       
       # Set the auto_vacuum PRAGMA using the given symbol (:none, :full, or
-      # :incremental).
+      # :incremental).  See pragma_set.  Consider using the :auto_vacuum
+      # Database option instead.
       def auto_vacuum=(value)
         value = AUTO_VACUUM.index(value) || (raise Error, "Invalid value for auto_vacuum option. Please specify one of :none, :full, :incremental.")
         pragma_set(:auto_vacuum, value)
@@ -31,12 +36,22 @@ module Sequel
       def database_type
         :sqlite
       end
+      
+      # Boolean signifying the value of the foreign_keys PRAGMA, or nil
+      # if not using SQLite 3.6.19+.
+      def foreign_keys
+        pragma_get(:foreign_keys).to_i == 1 if sqlite_version >= 30619
+      end
+      
+      # Set the foreign_keys PRAGMA using the given boolean value, if using
+      # SQLite 3.6.19+.  If not using 3.6.19+, no error is raised. See pragma_set.
+      # Consider using the :foreign_keys Database option instead.
+      def foreign_keys=(value)
+        pragma_set(:foreign_keys, !!value ? 'on' : 'off') if sqlite_version >= 30619
+      end
 
-      # Return a hash containing index information. Hash keys are index name symbols.
-      # Values are subhashes with two keys, :columns and :unique.  The value of :columns
-      # is an array of symbols of column names.  The value of :unique is true or false
-      # depending on if the index is unique.
-      def indexes(table)
+      # Use the index_list and index_info PRAGMAs to determine the indexes on the table.
+      def indexes(table, opts={})
         m = output_identifier_meth
         im = input_identifier_meth
         indexes = {}
@@ -61,13 +76,35 @@ module Sequel
       end
       
       # Set the value of the given PRAGMA to value.
+      #
+      # This method is not thread safe, and will not work correctly if there
+      # are multiple connections in the Database's connection pool. PRAGMA
+      # modifications should be done when the connection is created, using
+      # an option provided when creating the Database object.
       def pragma_set(name, value)
         execute_ddl("PRAGMA #{name} = #{value}")
       end
       
-      # SQLite supports savepoints
+      # The version of the server as an integer, where 3.6.19 = 30619.
+      # If the server version can't be determined, 0 is used.
+      def sqlite_version
+        return @sqlite_version if defined?(@sqlite_version)
+        @sqlite_version = begin
+          v = get{sqlite_version{}}
+          [10000, 100, 1].zip(v.split('.')).inject(0){|a, m| a + m[0] * Integer(m[1])}
+        rescue
+          0
+        end
+      end
+      
+      # SQLite supports CREATE TABLE IF NOT EXISTS syntax since 3.3.0.
+      def supports_create_table_if_not_exists?
+        sqlite_version >= 30300
+      end
+      
+      # SQLite 3.6.8+ supports savepoints. 
       def supports_savepoints?
-        true
+        sqlite_version >= 30608
       end
 
       # A symbol signifying the value of the synchronous PRAGMA.
@@ -75,7 +112,8 @@ module Sequel
         SYNCHRONOUS[pragma_get(:synchronous).to_i]
       end
       
-      # Set the synchronous PRAGMA using the given symbol (:off, :normal, or :full).
+      # Set the synchronous PRAGMA using the given symbol (:off, :normal, or :full). See pragma_set.
+      # Consider using the :synchronous Database option instead.
       def synchronous=(value)
         value = SYNCHRONOUS.index(value) || (raise Error, "Invalid value for synchronous option. Please specify one of :off, :normal, :full.")
         pragma_set(:synchronous, value)
@@ -86,8 +124,7 @@ module Sequel
       # Options:
       # * :server - Set the server to use.
       def tables(opts={})
-        m = output_identifier_meth
-        metadata_dataset.from(:sqlite_master).server(opts[:server]).filter(TABLES_FILTER).map{|r| m.call(r[:name])}
+        tables_and_views(TABLES_FILTER, opts)
       end
       
       # A symbol signifying the value of the temp_store PRAGMA.
@@ -95,12 +132,21 @@ module Sequel
         TEMP_STORE[pragma_get(:temp_store).to_i]
       end
       
-      # Set the temp_store PRAGMA using the given symbol (:default, :file, or :memory).
+      # Set the temp_store PRAGMA using the given symbol (:default, :file, or :memory). See pragma_set.
+      # Consider using the :temp_store Database option instead.
       def temp_store=(value)
         value = TEMP_STORE.index(value) || (raise Error, "Invalid value for temp_store option. Please specify one of :default, :file, :memory.")
         pragma_set(:temp_store, value)
       end
       
+      # Array of symbols specifying the view names in the current database.
+      #
+      # Options:
+      # * :server - Set the server to use.
+      def views(opts={})
+        tables_and_views(VIEWS_FILTER, opts)
+      end
+
       private
 
       # SQLite supports limited table modification.  You can add a column
@@ -148,6 +194,21 @@ module Sequel
         generator.is_a?(Schema::Generator) ? super : generator.map{|c| column_definition_sql(c)}.join(', ')
       end
 
+      # Array of PRAGMA SQL statements based on the Database options that should be applied to
+      # new connections.
+      def connection_pragmas
+        ps = []
+        v = typecast_value_boolean(opts.fetch(:foreign_keys, 1))
+        ps << "PRAGMA foreign_keys = #{v ? 1 : 0}"
+        [[:auto_vacuum, AUTO_VACUUM], [:synchronous, SYNCHRONOUS], [:temp_store, TEMP_STORE]].each do |prag, con|
+          if v = opts[prag]
+            raise(Error, "Value for PRAGMA #{prag} not supported, should be one of #{con.join(', ')}") unless v = con.index(v.to_sym)
+            ps << "PRAGMA #{prag} = #{v}"
+          end
+        end
+        ps
+      end
+
       # The array of column schema hashes, except for the ones given in opts[:except]
       def defined_columns_for(table, opts={})
         cols = parse_pragma(table, {})
@@ -159,9 +220,20 @@ module Sequel
           nono= Array(opts[:except]).compact.map{|n| n.to_s}
           cols.reject!{|c| nono.include? c[:name] }
         end
+
+        if foreign_keys
+          metadata_dataset.with_sql("PRAGMA foreign_key_list(?)", input_identifier_meth.call(table)).each do |row|
+            c = cols.find {|co| co[:name] == row[:from] } or next
+            c[:table] = row[:table]
+            c[:key] = row[:to]
+            c[:on_update] = on_delete_sql_to_sym(row[:on_update])
+            c[:on_delete] = on_delete_sql_to_sym(row[:on_delete])
+            # is there any way to get deferrable status?
+          end
+        end
         cols
       end
-      
+
       # Duplicate an existing table by creating a new table, copying all records
       # from the existing table into the new table, deleting the existing table
       # and renaming the new table to the existing table's name.
@@ -178,12 +250,18 @@ module Sequel
 
         qt = quote_schema_table(table)
         bt = quote_identifier(backup_table_name(qt))
-        [
+        a = [
            "CREATE TABLE #{bt}(#{def_columns_str})",
            "INSERT INTO #{bt}(#{dataset.send(:identifier_list, new_columns)}) SELECT #{dataset.send(:identifier_list, old_columns)} FROM #{qt}",
            "DROP TABLE #{qt}",
            "ALTER TABLE #{bt} RENAME TO #{qt}"
         ]
+        indexes(table).each do |name, h|
+          if (h[:columns].map{|x| x.to_s} - new_columns).empty?
+            a << alter_table_sql(table, h.merge(:op=>:add_index, :name=>name))
+          end
+        end
+        a
       end
 
       # SQLite folds unquoted identifiers to lowercase, so it shouldn't need to upcase identifiers on input.
@@ -195,6 +273,23 @@ module Sequel
       def identifier_output_method_default
         nil
       end
+      
+      # Does the reverse of on_delete_clause, eg. converts strings like +'SET NULL'+
+      # to symbols +:set_null+.
+      def on_delete_sql_to_sym str
+        case str
+        when 'RESTRICT'
+          :restrict
+        when 'CASCADE'
+          :cascade
+        when 'SET NULL'
+          :set_null
+        when 'SET DEFAULT'
+          :set_default
+        when 'NO ACTION'
+          :no_action
+        end
+      end
 
       # Parse the output of the table_info pragma
       def parse_pragma(table_name, opts)
@@ -224,6 +319,12 @@ module Sequel
         end
       end
       
+      # Backbone of the tables and views support.
+      def tables_and_views(filter, opts)
+        m = output_identifier_meth
+        metadata_dataset.from(:sqlite_master).server(opts[:server]).filter(filter).map{|r| m.call(r[:name])}
+      end
+      
       # SQLite uses the integer data type even for bignums.  This is because they
       # are both stored internally as text, and converted when returned from
       # the database.  Using an integer type instead of bigint makes it more likely
@@ -236,8 +337,26 @@ module Sequel
     # Instance methods for datasets that connect to an SQLite database
     module DatasetMethods
       SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'distinct columns from join where group having compounds order limit')
+      COMMA_SEPARATOR = ', '.freeze
       CONSTANT_MAP = {:CURRENT_DATE=>"date(CURRENT_TIMESTAMP, 'localtime')".freeze, :CURRENT_TIMESTAMP=>"datetime(CURRENT_TIMESTAMP, 'localtime')".freeze, :CURRENT_TIME=>"time(CURRENT_TIMESTAMP, 'localtime')".freeze}
-    
+
+      # Ugly hack.  Really, SQLite uses 0 for false and 1 for true
+      # but then you can't differentiate between integers and booleans.
+      # In filters, SQL::BooleanConstants are used more, while in other places
+      # the ruby true/false values are used more, so use 1/0 for SQL::BooleanConstants.
+      # The correct fix for this would require separate literalization paths for
+      # filters compared to other values, but that's more work than I want to do right now.
+      def boolean_constant_sql(constant)
+        case constant
+        when true
+          '1'
+        when false
+          '0'
+        else
+          super
+        end
+      end
+
       # SQLite does not support pattern matching via regular expressions.
       # SQLite is case insensitive (depending on pragma), so use LIKE for
       # ILIKE.
@@ -273,7 +392,7 @@ module Sequel
       end
       
       # HAVING requires GROUP BY on SQLite
-      def having(*cond, &block)
+      def having(*cond)
         raise(InvalidOperation, "Can only specify a HAVING clause on a grouped dataset") unless @opts[:group]
         super
       end
@@ -312,18 +431,44 @@ module Sequel
         "#{expression} AS #{literal(aliaz.to_s)}"
       end
       
+      # Special case when true or false is provided directly to filter.
+      def filter_expr(expr)
+        if block_given?
+          super
+        else
+          case expr
+          when true
+            1
+          when false
+            0
+          else
+            super
+          end
+        end
+      end
+      
+      # SQL fragment specifying a list of identifiers
+      def identifier_list(columns)
+        columns.map{|i| quote_identifier(i)}.join(COMMA_SEPARATOR)
+      end
+    
       # SQLite uses a preceding X for hex escaping strings
       def literal_blob(v)
         blob = ''
         v.each_byte{|x| blob << sprintf('%02x', x)}
         "X'#{blob}'"
       end
-      
+
       # SQLite does not support the SQL WITH clause
       def select_clause_methods
         SELECT_CLAUSE_METHODS
       end
       
+      # Support FOR SHARE locking when using the :share lock style.
+      def select_lock_sql(sql)
+        super unless @opts[:lock] == :update
+      end
+      
       # SQLite treats a DELETE with no WHERE clause as a TRUNCATE
       def _truncate_sql(table)
         "DELETE FROM #{table}"
