@@ -2,7 +2,7 @@
 
 # Top level module for Sequel
 #
-# There are some class methods that are added via metaprogramming, one for
+# There are some module methods that are added via metaprogramming, one for
 # each supported adapter.  For example:
 #
 #   DB = Sequel.sqlite # Memory database
@@ -17,65 +17,81 @@
 #
 #   Sequel.sqlite('blog.db'){|db| puts db[:users].count} 
 #
-# Sequel converts two digit years in Dates and DateTimes by default,
-# so 01/02/03 is interpreted at January 2nd, 2003, and 12/13/99 is interpreted
-# as December 13, 1999. You can override this to treat those dates as
-# January 2nd, 0003 and December 13, 0099, respectively, by setting: 
-#
-#   Sequel.convert_two_digit_years = false
-#
-# Sequel can use either Time or DateTime for times returned from the
-# database.  It defaults to Time.  To change it to DateTime, use:
-#
-#   Sequel.datetime_class = DateTime
-#
-# Sequel doesn't pay much attention to timezones by default, but you can set it
-# handle timezones if you want.  There are three separate timezone settings:
-#
-# * application_timezone - The timezone you want the application to use.  This is
-#   the timezone that incoming times from the database and typecasting are converted
-#   to.
-# * database_timezone - The timezone for storage in the database.  This is the
-#   timezone to which Sequel will convert timestamps before literalizing them
-#   for storage in the database.  It is also the timezone that Sequel will assume
-#   database timestamp values are already in (if they don't include an offset).
-# * typecast_timezone - The timezone that incoming data that Sequel needs to typecast
-#   is assumed to be already in (if they don't include an offset).
-#
-# You can set also set all three timezones to the same value at once via
-# Sequel.default_timezone=.
-#
-#   Sequel.application_timezone = :utc # or :local or nil
-#   Sequel.database_timezone = :utc # or :local or nil
-#   Sequel.typecast_timezone = :utc # or :local or nil
-#   Sequel.default_timezone = :utc # or :local or nil
-#
-# The only timezone values that are supported by default are :utc (convert to UTC),
-# :local (convert to local time), and nil (don't convert).  If you need to
-# convert to a specific timezone, or need the timezones being used to change based
-# on the environment (e.g. current user), you need to use an extension (and use
-# DateTime as the datetime_class).
-#
-# You can set the SEQUEL_NO_CORE_EXTENSIONS constant or environment variable to have
+# You can set the +SEQUEL_NO_CORE_EXTENSIONS+ constant or environment variable to have
 # Sequel not extend the core classes.
+#
+# For a more expanded introduction, see the {README}[link:files/README_rdoc.html].
+# For a quicker introduction, see the {cheat sheet}[link:files/doc/cheat_sheet_rdoc.html].
 module Sequel
   @convert_two_digit_years = true
   @datetime_class = Time
   @virtual_row_instance_eval = true
+  @require_thread = nil
+  
+  # Mutex used to protect file loading/requireing
+  @require_mutex = Mutex.new
   
   class << self
-    attr_accessor :convert_two_digit_years, :datetime_class, :virtual_row_instance_eval
+    # Sequel converts two digit years in <tt>Date</tt>s and <tt>DateTime</tt>s by default,
+    # so 01/02/03 is interpreted at January 2nd, 2003, and 12/13/99 is interpreted
+    # as December 13, 1999. You can override this to treat those dates as
+    # January 2nd, 0003 and December 13, 0099, respectively, by:
+    #
+    #   Sequel.convert_two_digit_years = false
+    attr_accessor :convert_two_digit_years
+
+    # Sequel can use either +Time+ or +DateTime+ for times returned from the
+    # database.  It defaults to +Time+.  To change it to +DateTime+:
+    #
+    #   Sequel.datetime_class = DateTime
+    #
+    # For ruby versions less than 1.9.2, +Time+ has a limited range (1901 to
+    # 2038), so if you use datetimes out of that range, you need to switch
+    # to +DateTime+.  Also, before 1.9.2, +Time+ can only handle local and UTC
+    # times, not other timezones.  Note that +Time+ and +DateTime+ objects
+    # have a different API, and in cases where they implement the same methods,
+    # they often implement them differently (e.g. + using seconds on +Time+ and
+    # days on +DateTime+).
+    attr_accessor :datetime_class
+
+    # For backwards compatibility, has no effect.
+    attr_accessor :virtual_row_instance_eval
+    
+    # Alias to the standard version of require
+    alias k_require require
+
+    private
+
+    # Make thread safe requiring reentrant to prevent deadlocks.
+    def check_requiring_thread
+      t = Thread.current
+      return(yield) if @require_thread == t
+      @require_mutex.synchronize do
+        begin
+          @require_thread = t 
+          yield
+        ensure
+          @require_thread = nil
+        end
+      end
+    end
   end
 
   # Returns true if the passed object could be a specifier of conditions, false otherwise.
-  # Currently, Sequel considers hashes and arrays of all two pairs as
+  # Currently, Sequel considers hashes and arrays of two element arrays as
   # condition specifiers.
+  #
+  #   Sequel.condition_specifier?({}) # => true
+  #   Sequel.condition_specifier?([[1, 2]]) # => true
+  #   Sequel.condition_specifier?([]) # => false
+  #   Sequel.condition_specifier?([1]) # => false
+  #   Sequel.condition_specifier?(1) # => false
   def self.condition_specifier?(obj)
     case obj
     when Hash
       true
     when Array
-      !obj.empty? && obj.all?{|i| (Array === i) && (i.length == 2)}
+      !obj.empty? && !obj.is_a?(SQL::ValueList) && obj.all?{|i| (Array === i) && (i.length == 2)}
     else
       false
     end
@@ -92,17 +108,20 @@ module Sequel
   #   DB = Sequel.connect('postgres://user:password@host:port/database_name')
   #   DB = Sequel.connect('sqlite:///blog.db', :max_connections=>10)
   #
-  # If a block is given, it is passed the opened Database object, which is
+  # If a block is given, it is passed the opened +Database+ object, which is
   # closed when the block exits.  For example:
   #
   #   Sequel.connect('sqlite://blog.db'){|db| puts db[:users].count}  
+  # 
+  # For details, see the {"Connecting to a Database" guide}[link:files/doc/opening_databases_rdoc.html].
+  # To set up a master/slave or sharded database connection, see the {"Master/Slave Databases and Sharding" guide}[link:files/doc/sharding_rdoc.html].
   def self.connect(*args, &block)
     Database.connect(*args, &block)
   end
   
-  # Convert the exception to the given class.  The given class should be
-  # Sequel::Error or a subclass.  Returns an instance of klass with
-  # the message and backtrace of exception.
+  # Convert the +exception+ to the given class.  The given class should be
+  # <tt>Sequel::Error</tt> or a subclass.  Returns an instance of +klass+ with
+  # the message and backtrace of +exception+.
   def self.convert_exception_class(exception, klass)
     return exception if exception.is_a?(klass)
     e = klass.new("#{exception.class}: #{exception.message}")
@@ -111,13 +130,18 @@ module Sequel
     e
   end
 
-  # Load all Sequel extensions given.  Only loads extensions included in this
-  # release of Sequel, doesn't load external extensions.
+  # Load all Sequel extensions given.  Extensions are just files that exist under
+  # <tt>sequel/extensions</tt> in the load path, and are just required.  Generally,
+  # extensions modify the behavior of +Database+ and/or +Dataset+, but Sequel ships
+  # with some extensions that modify other classes that exist for backwards compatibility.
+  # In some cases, requiring an extension modifies classes directly, and in others,
+  # it just loads a module that you can extend other classes with.  Consult the documentation
+  # for each extension you plan on using for usage.
   #
   #   Sequel.extension(:schema_dumper)
   #   Sequel.extension(:pagination, :query)
   def self.extension(*extensions)
-    require(extensions, 'extensions')
+    extensions.each{|e| tsk_require "sequel/extensions/#{e}"}
   end
   
   # Set the method to call on identifiers going into the database.  This affects
@@ -158,15 +182,17 @@ module Sequel
   def self.quote_identifiers=(value)
     Database.quote_identifiers = value
   end
-
-  # Require all given files which should be in the same or a subdirectory of
-  # this file.  If a subdir is given, assume all files are in that subdir.
+  
+  # Require all given +files+ which should be in the same or a subdirectory of
+  # this file.  If a +subdir+ is given, assume all +files+ are in that subdir.
+  # This is used to ensure that the files loaded are from the same version of
+  # Sequel as this file.
   def self.require(files, subdir=nil)
     Array(files).each{|f| super("#{File.dirname(__FILE__)}/#{"#{subdir}/" if subdir}#{f}")}
   end
   
   # Set whether to set the single threaded mode for all databases by default. By default,
-  # Sequel uses a threadsafe connection pool, which isn't as fast as the
+  # Sequel uses a thread-safe connection pool, which isn't as fast as the
   # single threaded connection pool.  If your program will only have one thread,
   # and speed is a priority, you may want to set this to true:
   #
@@ -175,42 +201,61 @@ module Sequel
     Database.single_threaded = value
   end
 
-  # Converts the given string into a Date object.
-  def self.string_to_date(s)
+  # Converts the given +string+ into a +Date+ object.
+  #
+  #   Sequel.string_to_date('2010-09-10') # Date.civil(2010, 09, 10)
+  def self.string_to_date(string)
     begin
-      Date.parse(s, Sequel.convert_two_digit_years)
+      Date.parse(string, Sequel.convert_two_digit_years)
     rescue => e
       raise convert_exception_class(e, InvalidValue)
     end
   end
 
-  # Converts the given string into a Time or DateTime object, depending on the
-  # value of Sequel.datetime_class.
-  def self.string_to_datetime(s)
+  # Converts the given +string+ into a +Time+ or +DateTime+ object, depending on the
+  # value of <tt>Sequel.datetime_class</tt>.
+  #
+  #   Sequel.string_to_datetime('2010-09-10 10:20:30') # Time.local(2010, 09, 10, 10, 20, 30)
+  def self.string_to_datetime(string)
     begin
       if datetime_class == DateTime
-        DateTime.parse(s, convert_two_digit_years)
+        DateTime.parse(string, convert_two_digit_years)
       else
-        datetime_class.parse(s)
+        datetime_class.parse(string)
       end
     rescue => e
       raise convert_exception_class(e, InvalidValue)
     end
   end
 
-  # Converts the given string into a Time object.
-  def self.string_to_time(s)
+  # Converts the given +string+ into a +Time+ object.
+  #
+  #   Sequel.string_to_datetime('10:20:30') # Time.parse('10:20:30')
+  def self.string_to_time(string)
     begin
-      Time.parse(s)
+      Time.parse(string)
     rescue => e
       raise convert_exception_class(e, InvalidValue)
     end
   end
 
+  # Same as Sequel.require, but wrapped in a mutex in order to be thread safe.
+  def self.ts_require(*args)
+    check_requiring_thread{require(*args)}
+  end
+  
+  # Same as Kernel.require, but wrapped in a mutex in order to be thread safe.
+  def self.tsk_require(*args)
+    check_requiring_thread{k_require(*args)}
+  end
+
   # If the supplied block takes a single argument,
-  # yield a new SQL::VirtualRow instance to the block
+  # yield a new <tt>SQL::VirtualRow</tt> instance to the block
   # argument.  Otherwise, evaluate the block in the context of a new
-  # SQL::VirtualRow instance.
+  # <tt>SQL::VirtualRow</tt> instance.
+  #
+  #   Sequel.virtual_row{a} # Sequel::SQL::Identifier.new(:a)
+  #   Sequel.virtual_row{|o| o.a{}} # Sequel::SQL::Function.new(:a)
   def self.virtual_row(&block)
     vr = SQL::VirtualRow.new
     case block.arity
@@ -247,9 +292,7 @@ module Sequel
 
   private_class_method :adapter_method, :def_adapter_method
   
-  require(%w"metaprogramming sql connection_pool exceptions dataset database timezones version")
-  require(%w"schema_generator schema_methods schema_sql", 'database')
-  require(%w"actions convenience features graph prepared_statements query sql", 'dataset')
+  require(%w"metaprogramming sql connection_pool exceptions dataset database timezones ast_transformer version")
   require('core_sql') if !defined?(::SEQUEL_NO_CORE_EXTENSIONS) && !ENV.has_key?('SEQUEL_NO_CORE_EXTENSIONS')
 
   # Add the database adapter class methods to Sequel via metaprogramming
