@@ -1,5 +1,11 @@
 module Sequel 
   class Dataset
+    # ---------------------
+    # :section: Methods related to prepared statements or bound variables
+    # On some adapters, these use native prepared statements and bound variables, on others
+    # support is emulated.  For details, see the {"Prepared Statements/Bound Variables" guide}[link:files/doc/prepared_statements_rdoc.html].
+    # ---------------------
+    
     PREPARED_ARG_PLACEHOLDER = LiteralString.new('?').freeze
     
     # Default implementation of the argument mapper to allow
@@ -75,6 +81,8 @@ module Sequel
           select_sql
         when :first
           clone(:limit=>1).select_sql
+        when :insert_select
+          clone(:returning=>nil).insert_sql(*@prepared_modify_values)
         when :insert
           insert_sql(*@prepared_modify_values)
         when :update
@@ -89,8 +97,8 @@ module Sequel
       # and they are substituted using prepared_arg.
       def literal_symbol(v)
         if @opts[:bind_vars] and match = PLACEHOLDER_RE.match(v.to_s)
-          v2 = prepared_arg(match[1].to_sym)
-          v2 ? literal(v2) : v
+          s = match[1].to_sym
+          prepared_arg?(s) ? literal(prepared_arg(s)) : v
         else
           super
         end
@@ -112,6 +120,9 @@ module Sequel
         case @prepared_type
         when :select, :all
           all(&block)
+        when :insert_select
+          meta_def(:select_sql){prepared_sql}
+          first
         when :first
           first
         when :insert
@@ -130,6 +141,11 @@ module Sequel
         @opts[:bind_vars][k]
       end
 
+      # Whether there is a bound value for the given key.
+      def prepared_arg?(k)
+        @opts[:bind_vars].has_key?(k)
+      end
+
       # Use a clone of the dataset extended with prepared statement
       # support and using the same argument hash so that you can use
       # bind variables/prepared arguments in subselects.
@@ -165,11 +181,20 @@ module Sequel
         prepared_args << k
         prepared_arg_placeholder
       end
+      
+      # Always assume there is a prepared arg in the argument mapper.
+      def prepared_arg?(k)
+        true
+      end
     end
     
     # Set the bind variables to use for the call.  If bind variables have
     # already been set for this dataset, they are updated with the contents
     # of bind_vars.
+    #
+    #   DB[:table].filter(:id=>:$id).bind(:id=>1).call(:first)
+    #   # SELECT * FROM table WHERE id = ? LIMIT 1 -- (1)
+    #   # => {:id=>1}
     def bind(bind_vars={})
       clone(:bind_vars=>@opts[:bind_vars] ? @opts[:bind_vars].merge(bind_vars) : bind_vars)
     end
@@ -179,6 +204,10 @@ module Sequel
     # specified in the hash.  values is a hash of passed to
     # insert or update (if one of those types is used),
     # which may contain placeholders.
+    #
+    #   DB[:table].filter(:id=>:$id).call(:first, :id=>1)
+    #   # SELECT * FROM table WHERE id = ? LIMIT 1 -- (1)
+    #   # => {:id=>1}
     def call(type, bind_variables={}, *values, &block)
       prepare(type, nil, *values).call(bind_variables, &block)
     end
@@ -189,9 +218,13 @@ module Sequel
     # do substitution.  The prepared statement is also stored in
     # the associated database.  The following usage is identical:
     #
-    #   ps = prepare(:select, :select_by_name)
+    #   ps = DB[:table].filter(:name=>:$name).prepare(:first, :select_by_name)
+    #
     #   ps.call(:name=>'Blah')
-    #   db.call(:select_by_name, :name=>'Blah')
+    #   # SELECT * FROM table WHERE name = ? -- ('Blah')
+    #   # => {:id=>1, :name=>'Blah'}
+    #
+    #   DB.call(:select_by_name, :name=>'Blah') # Same thing
     def prepare(type, name=nil, *values)
       ps = to_prepared_statement(type, values)
       db.prepared_statements[name] = ps if name
