@@ -1,4 +1,4 @@
-require File.join(File.dirname(__FILE__), 'spec_helper.rb')
+require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
 
 unless defined?(SQLITE_DB)
   SQLITE_URL = 'sqlite:/' unless defined? SQLITE_URL
@@ -6,11 +6,13 @@ unless defined?(SQLITE_DB)
 end
 INTEGRATION_DB = SQLITE_DB unless defined?(INTEGRATION_DB)
 
-context "An SQLite database" do
+describe "An SQLite database" do
   before do
     @db = SQLITE_DB
+    @fk = @db.foreign_keys
   end
   after do
+    @db.foreign_keys = @fk
     Sequel.datetime_class = Time
   end
 
@@ -36,6 +38,34 @@ context "An SQLite database" do
     end
   end
   
+  specify "should provide the SQLite version as an integer" do
+    @db.sqlite_version.should be_a_kind_of(Integer)
+  end
+  
+  specify "should support setting and getting the foreign_keys pragma" do
+    (@db.sqlite_version >= 30619 ? [true, false] : [nil]).should include(@db.foreign_keys)
+    @db.foreign_keys = true
+    @db.foreign_keys = false
+  end
+  
+  if SQLITE_DB.sqlite_version >= 30619
+    specify "should enforce foreign key integrity if foreign_keys pragma is set" do
+      @db.foreign_keys = true
+      @db.create_table!(:fk){primary_key :id; foreign_key :parent_id, :fk}
+      @db[:fk].insert(1, nil)
+      @db[:fk].insert(2, 1)
+      @db[:fk].insert(3, 3)
+      proc{@db[:fk].insert(4, 5)}.should raise_error(Sequel::Error)
+    end
+  end
+  
+  specify "should not enforce foreign key integrity if foreign_keys pragma is unset" do
+    @db.foreign_keys = false
+    @db.create_table!(:fk){primary_key :id; foreign_key :parent_id, :fk}
+    @db[:fk].insert(1, 2)
+    @db[:fk].all.should == [{:id=>1, :parent_id=>2}]
+  end
+  
   specify "should provide a list of existing tables" do
     @db.drop_table(:testing) rescue nil
     @db.tables.should be_a_kind_of(Array)
@@ -68,17 +98,16 @@ context "An SQLite database" do
     proc {@db.temp_store = :invalid}.should raise_error(Sequel::Error)
   end
   
-  cspecify "should support timestamps and datetimes and respect datetime_class", :do, :jdbc, :amalgalite do
+  cspecify "should support timestamps and datetimes and respect datetime_class", :do, :jdbc, :amalgalite, :swift do
     @db.create_table!(:time){timestamp :t; datetime :d}
     t1 = Time.at(1)
-    @db[:time] << {:t => t1, :d => t1.to_i}
-    @db[:time] << {:t => t1.to_i, :d => t1}
-    @db[:time].map(:t).should == [t1, t1]
-    @db[:time].map(:d).should == [t1, t1]
+    @db[:time] << {:t => t1, :d => t1}
+    @db[:time].map(:t).should == [t1]
+    @db[:time].map(:d).should == [t1]
     Sequel.datetime_class = DateTime
     t2 = Sequel.string_to_datetime(t1.iso8601)
-    @db[:time].map(:t).should == [t2, t2]
-    @db[:time].map(:d).should == [t2, t2]
+    @db[:time].map(:t).should == [t2]
+    @db[:time].map(:d).should == [t2]
   end
   
   specify "should support sequential primary keys" do
@@ -99,7 +128,7 @@ context "An SQLite database" do
   end
 end
 
-context "An SQLite dataset" do
+describe "An SQLite dataset" do
   before do
     @d = SQLITE_DB[:items]
   end
@@ -119,7 +148,7 @@ context "An SQLite dataset" do
   end
 end
 
-context "An SQLite numeric column" do
+describe "An SQLite numeric column" do
   specify "should handle and return BigDecimal values" do
     SQLITE_DB.create_table!(:d){numeric :d}
     d = SQLITE_DB[:d]
@@ -133,7 +162,7 @@ context "An SQLite numeric column" do
   end
 end
 
-context "An SQLite dataset AS clause" do
+describe "An SQLite dataset AS clause" do
   specify "should use a string literal for :col___alias" do
     SQLITE_DB.literal(:c___a).should == "c AS 'a'"
   end
@@ -159,7 +188,7 @@ context "An SQLite dataset AS clause" do
   end
 end
 
-context "SQLite::Dataset#delete" do
+describe "SQLite::Dataset#delete" do
   before do
     SQLITE_DB.create_table! :items do
       primary_key :id
@@ -191,7 +220,7 @@ context "SQLite::Dataset#delete" do
   end
 end
 
-context "SQLite::Dataset#update" do
+describe "SQLite::Dataset#update" do
   before do
     SQLITE_DB.create_table! :items do
       primary_key :id
@@ -214,7 +243,7 @@ context "SQLite::Dataset#update" do
   end
 end
 
-context "SQLite dataset" do
+describe "SQLite dataset" do
   before do
     SQLITE_DB.create_table! :test do
       primary_key :id
@@ -253,7 +282,7 @@ context "SQLite dataset" do
   end
 end
 
-context "A SQLite database" do
+describe "A SQLite database" do
   before do
     @db = SQLITE_DB
     @db.create_table! :test2 do
@@ -297,13 +326,41 @@ context "A SQLite database" do
     @db[:test3] << { :name => "foo", :value => 2}
     @db[:test3] << { :name => "foo", :value => 3}
     @db[:test3].filter(:id => 2).delete
-    
+
     @db.drop_column :test3, :value
 
     @db['PRAGMA table_info(?)', :test3][:id][:pk].to_i.should == 1
     @db[:test3].select(:id).all.should == [{:id => 1}, {:id => 3}]
   end
 
+  if SQLITE_DB.foreign_keys
+    specify "should keep foreign keys when dropping a column" do
+      @db.create_table! :test do
+        primary_key :id
+        String :name
+        Integer :value
+      end
+      @db.create_table! :test3 do
+        String :name
+        Integer :value
+        foreign_key :test_id, :test, :on_delete => :set_null, :on_update => :cascade
+      end
+
+      @db[:test3].insert(:name => "abc", :test_id => @db[:test].insert(:name => "foo", :value => 3))
+      @db[:test3].insert(:name => "def", :test_id => @db[:test].insert(:name => "bar", :value => 4))
+
+      @db.drop_column :test3, :value
+
+      @db[:test].filter(:name => 'bar').delete
+      @db[:test3][:name => 'def'][:test_id].should be_nil
+
+      @db[:test].filter(:name => 'foo').update(:id=>100)
+      @db[:test3][:name => 'abc'][:test_id].should == 100
+
+      @db.drop_table :test, :test3
+    end
+  end
+
   specify "should support rename_column operations" do
     @db[:test2].delete
     @db.add_column :test2, :xyz, :text
@@ -354,7 +411,7 @@ context "A SQLite database" do
   
   specify "should choose a temporary table name that isn't already used when dropping or renaming columns" do
     sqls = []
-    @db.loggers << (l=Class.new{define_method(:info){|sql| sqls << sql}}.new)
+    @db.loggers << (l=Class.new{%w'info error'.each{|m| define_method(m){|sql| sqls << sql}}}.new)
     @db.create_table! :test3 do
       Integer :h
       Integer :i
@@ -401,4 +458,13 @@ context "A SQLite database" do
     @db.add_index :test2, :value, :unique => true
     @db.drop_index :test2, :value
   end
+
+  specify "should keep applicable indexes when emulating schema methods" do
+    @db.create_table!(:a){Integer :a; Integer :b}
+    @db.add_index :a, :a
+    @db.add_index :a, :b
+    @db.add_index :a, [:b, :a]
+    @db.drop_column :a, :b
+    @db.indexes(:a).should == {:a_a_index=>{:unique=>false, :columns=>[:a]}}
+  end
 end  
