@@ -1,6 +1,6 @@
-require File.join(File.dirname(__FILE__), 'spec_helper')
+require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper')
 
-context "A new Database" do
+describe "A new Database" do
   before do
     @db = Sequel::Database.new(1 => 2, :logger => 3)
   end
@@ -11,7 +11,8 @@ context "A new Database" do
   end
   
   specify "should receive options" do
-    @db.opts.should == {1 => 2, :logger => 3}  
+    @db.opts[1].should == 2
+    @db.opts[:logger].should == 3  
   end
   
   specify "should set the logger from opts[:logger] and opts[:loggers]" do
@@ -22,6 +23,11 @@ context "A new Database" do
     Sequel::Database.new(1 => 2, :logger => [4], :loggers => [3]).loggers.should == [4,3]
   end
   
+  specify "should set the sql_log_level from opts[:sql_log_level]" do
+    db = Sequel::Database.new(1 => 2, :sql_log_level=>:debug).sql_log_level.should == :debug
+    db = Sequel::Database.new(1 => 2, :sql_log_level=>'debug').sql_log_level.should == :debug
+  end
+  
   specify "should create a connection pool" do
     @db.pool.should be_a_kind_of(Sequel::ConnectionPool)
     @db.pool.max_size.should == 4
@@ -37,17 +43,17 @@ context "A new Database" do
   end
 
   specify "should respect the :single_threaded option" do
-    db = Sequel::Database.new(:single_threaded=>true)
-    db.pool.should be_a_kind_of(Sequel::SingleThreadedPool)
-    db = Sequel::Database.new(:single_threaded=>'t')
-    db.pool.should be_a_kind_of(Sequel::SingleThreadedPool)
-    db = Sequel::Database.new(:single_threaded=>'1')
-    db.pool.should be_a_kind_of(Sequel::SingleThreadedPool)
-    db = Sequel::Database.new(:single_threaded=>false)
+    db = Sequel::Database.new(:single_threaded=>true){123}
+    db.pool.should be_a_kind_of(Sequel::SingleConnectionPool)
+    db = Sequel::Database.new(:single_threaded=>'t'){123}
+    db.pool.should be_a_kind_of(Sequel::SingleConnectionPool)
+    db = Sequel::Database.new(:single_threaded=>'1'){123}
+    db.pool.should be_a_kind_of(Sequel::SingleConnectionPool)
+    db = Sequel::Database.new(:single_threaded=>false){123}
     db.pool.should be_a_kind_of(Sequel::ConnectionPool)
-    db = Sequel::Database.new(:single_threaded=>'f')
+    db = Sequel::Database.new(:single_threaded=>'f'){123}
     db.pool.should be_a_kind_of(Sequel::ConnectionPool)
-    db = Sequel::Database.new(:single_threaded=>'0')
+    db = Sequel::Database.new(:single_threaded=>'0'){123}
     db.pool.should be_a_kind_of(Sequel::ConnectionPool)
   end
 
@@ -185,30 +191,128 @@ context "A new Database" do
     db.should be_a_kind_of(Sequel::Database)
     db.opts[:uri].should == 'do:test://host/db_name'
   end
+
+  specify "should populate :adapter option when using connection string" do
+    Sequel::Database.should_receive(:adapter_class).once.with(:do).and_return(Sequel::Database)
+    Sequel.connect('do:test://host/db_name').opts[:adapter] == :do
+  end
 end
 
-context "Database#disconnect" do
+describe "Database#disconnect" do
   specify "should call pool.disconnect" do
     d = Sequel::Database.new
     p = d.pool
-    p.should_receive(:disconnect).once.and_return(2)
+    p.should_receive(:disconnect).once.with({}).and_return(2)
     d.disconnect.should == 2
   end
 end
 
-context "Sequel.extension" do
+describe "Sequel.extension" do
   specify "should attempt to load the given extension" do
     proc{Sequel.extension :blah}.should raise_error(LoadError)
   end
 end
 
-context "Database#connect" do
-  specify "should raise NotImplementedError" do
-    proc {Sequel::Database.new.connect}.should raise_error(NotImplementedError)
+describe "Database#connect" do
+  specify "should raise Sequel::NotImplemented" do
+    proc {Sequel::Database.new.connect(:default)}.should raise_error(Sequel::NotImplemented)
   end
 end
 
-context "Database#uri" do
+describe "Database#log_info" do
+  before do
+    @o = Object.new
+    def @o.logs; @logs || []; end
+    def @o.to_ary; [self]; end
+    def @o.method_missing(*args); (@logs ||= []) << args; end
+    @db = Sequel::Database.new(:logger=>@o)
+  end
+
+  specify "should log message at info level to all loggers" do
+    @db.log_info('blah')
+    @o.logs.should == [[:info, 'blah']]
+  end
+
+  specify "should log message with args at info level to all loggers" do
+    @db.log_info('blah', [1, 2])
+    @o.logs.should == [[:info, 'blah; [1, 2]']]
+  end
+end
+
+describe "Database#log_yield" do
+  before do
+    @o = Object.new
+    def @o.logs; @logs || []; end
+    def @o.warn(*args); (@logs ||= []) << [:warn] + args; end
+    def @o.method_missing(*args); (@logs ||= []) << args; end
+    def @o.to_ary; [self]; end
+    @db = Sequel::Database.new(:logger=>@o)
+  end
+
+  specify "should yield to the passed block" do
+    a = nil
+    @db.log_yield('blah'){a = 1}
+    a.should == 1
+  end
+
+  specify "should raise an exception if a block is not passed" do
+    proc{@db.log_yield('blah')}.should raise_error
+  end
+
+  specify "should log message with duration at info level to all loggers" do
+    @db.log_yield('blah'){}
+    @o.logs.length.should == 1
+    @o.logs.first.length.should == 2
+    @o.logs.first.first.should == :info
+    @o.logs.first.last.should =~ /\A\(\d\.\d{6}s\) blah\z/ 
+  end
+
+  specify "should respect sql_log_level setting" do
+    @db.sql_log_level = :debug
+    @db.log_yield('blah'){}
+    @o.logs.length.should == 1
+    @o.logs.first.length.should == 2
+    @o.logs.first.first.should == :debug
+    @o.logs.first.last.should =~ /\A\(\d\.\d{6}s\) blah\z/ 
+  end
+
+  specify "should log message with duration at warn level if duration greater than log_warn_duration" do
+    @db.log_warn_duration = 0
+    @db.log_yield('blah'){}
+    @o.logs.length.should == 1
+    @o.logs.first.length.should == 2
+    @o.logs.first.first.should == :warn
+    @o.logs.first.last.should =~ /\A\(\d\.\d{6}s\) blah\z/ 
+  end
+
+  specify "should log message with duration at info level if duration less than log_warn_duration" do
+    @db.log_warn_duration = 1000
+    @db.log_yield('blah'){}
+    @o.logs.length.should == 1
+    @o.logs.first.length.should == 2
+    @o.logs.first.first.should == :info
+    @o.logs.first.last.should =~ /\A\(\d\.\d{6}s\) blah\z/ 
+  end
+
+  specify "should log message at error level if block raises an error" do
+    @db.log_warn_duration = 0
+    proc{@db.log_yield('blah'){raise Sequel::Error, 'adsf'}}.should raise_error
+    @o.logs.length.should == 1
+    @o.logs.first.length.should == 2
+    @o.logs.first.first.should == :error
+    @o.logs.first.last.should =~ /\ASequel::Error: adsf: blah\z/ 
+  end
+
+  specify "should include args with message if args passed" do
+    @db.log_yield('blah', [1, 2]){}
+    @o.logs.length.should == 1
+    @o.logs.first.length.should == 2
+    @o.logs.first.first.should == :info
+    @o.logs.first.last.should =~ /\A\(\d\.\d{6}s\) blah; \[1, 2\]\z/ 
+  end
+end
+
+describe "Database#uri" do
   before do
     @c = Class.new(Sequel::Database) do
       set_adapter_scheme :mau
@@ -226,7 +330,7 @@ context "Database#uri" do
   end
 end
 
-context "Database.adapter_scheme" do
+describe "Database.adapter_scheme and #adapter_scheme" do
   specify "should return the database schema" do
     Sequel::Database.adapter_scheme.should be_nil
 
@@ -235,10 +339,11 @@ context "Database.adapter_scheme" do
     end
     
     @c.adapter_scheme.should == :mau
+    @c.new({}).adapter_scheme.should == :mau
   end
 end
 
-context "Database#dataset" do
+describe "Database#dataset" do
   before do
     @db = Sequel::Database.new
     @ds = @db.dataset
@@ -279,14 +384,32 @@ context "Database#dataset" do
   end
 end
 
-context "Database#execute" do
-  specify "should raise NotImplementedError" do
-    proc {Sequel::Database.new.execute('blah blah')}.should raise_error(NotImplementedError)
-    proc {Sequel::Database.new << 'blah blah'}.should raise_error(NotImplementedError)
+describe "Database#execute" do
+  specify "should raise Sequel::NotImplemented" do
+    proc {Sequel::Database.new.execute('blah blah')}.should raise_error(Sequel::NotImplemented)
+    proc {Sequel::Database.new << 'blah blah'}.should raise_error(Sequel::NotImplemented)
   end
 end
 
-context "Database#<< and run" do
+describe "Database#tables" do
+  specify "should raise Sequel::NotImplemented" do
+    proc {Sequel::Database.new.tables}.should raise_error(Sequel::NotImplemented)
+  end
+end
+
+describe "Database#views" do
+  specify "should raise Sequel::NotImplemented" do
+    proc {Sequel::Database.new.views}.should raise_error(Sequel::NotImplemented)
+  end
+end
+
+describe "Database#indexes" do
+  specify "should raise Sequel::NotImplemented" do
+    proc {Sequel::Database.new.indexes(:table)}.should raise_error(Sequel::NotImplemented)
+  end
+end
+
+describe "Database#<< and run" do
   before do
     sqls = @sqls = []
     @c = Class.new(Sequel::Database) do
@@ -313,10 +436,9 @@ context "Database#<< and run" do
   end
 end
 
-context "Database#synchronize" do
+describe "Database#synchronize" do
   before do
-    @db = Sequel::Database.new(:max_connections => 1)
-    @db.pool.connection_proc = proc {12345}
+    @db = Sequel::Database.new(:max_connections => 1){12345}
   end
   
   specify "should wrap the supplied block in pool.hold" do
@@ -337,14 +459,12 @@ context "Database#synchronize" do
   end
 end
 
-context "Database#test_connection" do
+describe "Database#test_connection" do
   before do
-    @db = Sequel::Database.new
-    @test = nil
-    @db.pool.connection_proc = proc {@test = rand(100)}
+    @db = Sequel::Database.new{@test = rand(100)}
   end
   
-  specify "should call pool#hold" do
+  specify "should pool#hold" do
     @db.test_connection
     @test.should_not be_nil
   end
@@ -352,6 +472,10 @@ context "Database#test_connection" do
   specify "should return true if successful" do
     @db.test_connection.should be_true
   end
+
+  specify "should raise an error if the attempting to connect raises an error" do
+    proc{Sequel::Database.new{raise Sequel::Error, 'blah'}.test_connection}.should raise_error(Sequel::Error)
+  end
 end
 
 class DummyDataset < Sequel::Dataset
@@ -376,7 +500,7 @@ class DummyDatabase < Sequel::Database
   end
 end
 
-context "Database#create_table" do
+describe "Database#create_table" do
   before do
     @db = DummyDatabase.new
   end
@@ -405,9 +529,17 @@ context "Database#create_table" do
       'CREATE UNIQUE INDEX test_tmp_name_index ON test_tmp (name)'
     ]
   end
+
+  specify "should not use default schema when creating a temporary table" do
+    @db.default_schema = :foo
+    @db.create_table :test_tmp, :temp => true do
+      column :name, :text
+    end
+    @db.sqls.should == ['CREATE TEMPORARY TABLE test_tmp (name text)']
+  end
 end
 
-context "Database#alter_table" do
+describe "Database#alter_table" do
   before do
     @db = DummyDatabase.new
   end
@@ -425,7 +557,7 @@ context "Database#alter_table" do
     end
     
     @db.sqls.should == [
-      'ALTER TABLE xyz ADD COLUMN aaa text UNIQUE NOT NULL',
+      'ALTER TABLE xyz ADD COLUMN aaa text NOT NULL UNIQUE',
       'ALTER TABLE xyz DROP COLUMN bbb',
       'ALTER TABLE xyz RENAME COLUMN ccc TO ddd',
       'ALTER TABLE xyz ALTER COLUMN eee TYPE integer',
@@ -437,7 +569,7 @@ context "Database#alter_table" do
   end
 end
 
-context "Database#add_column" do
+describe "Database#add_column" do
   before do
     @db = DummyDatabase.new
   end
@@ -450,7 +582,7 @@ context "Database#add_column" do
   end
 end
 
-context "Database#drop_column" do
+describe "Database#drop_column" do
   before do
     @db = DummyDatabase.new
   end
@@ -463,7 +595,7 @@ context "Database#drop_column" do
   end
 end
 
-context "Database#rename_column" do
+describe "Database#rename_column" do
   before do
     @db = DummyDatabase.new
   end
@@ -476,7 +608,7 @@ context "Database#rename_column" do
   end
 end
 
-context "Database#set_column_type" do
+describe "Database#set_column_type" do
   before do
     @db = DummyDatabase.new
   end
@@ -489,7 +621,7 @@ context "Database#set_column_type" do
   end
 end
 
-context "Database#set_column_default" do
+describe "Database#set_column_default" do
   before do
     @db = DummyDatabase.new
   end
@@ -502,7 +634,7 @@ context "Database#set_column_default" do
   end
 end
 
-context "Database#add_index" do
+describe "Database#add_index" do
   before do
     @db = DummyDatabase.new
   end
@@ -522,7 +654,7 @@ context "Database#add_index" do
   end
 end
 
-context "Database#drop_index" do
+describe "Database#drop_index" do
   before do
     @db = DummyDatabase.new
   end
@@ -542,7 +674,7 @@ class Dummy2Database < Sequel::Database
   def transaction; yield; end
 end
 
-context "Database#drop_table" do
+describe "Database#drop_table" do
   before do
     @db = DummyDatabase.new
   end
@@ -562,7 +694,7 @@ context "Database#drop_table" do
   end
 end
 
-context "Database#rename_table" do
+describe "Database#rename_table" do
   before do
     @db = DummyDatabase.new
   end
@@ -573,7 +705,7 @@ context "Database#rename_table" do
   end
 end
 
-context "Database#table_exists?" do
+describe "Database#table_exists?" do
   specify "should try to select the first record from the table's dataset" do
     db2 = DummyDatabase.new
     db2.table_exists?(:a).should be_false
@@ -591,17 +723,39 @@ class Dummy3Database < Sequel::Database
   end
 end
 
-context "Database#transaction" do
+describe "Database#transaction" do
   before do
-    @db = Dummy3Database.new
-    @db.pool.connection_proc = proc {Dummy3Database::DummyConnection.new(@db)}
+    @db = Dummy3Database.new{Dummy3Database::DummyConnection.new(@db)}
   end
   
   specify "should wrap the supplied block with BEGIN + COMMIT statements" do
-    @db.transaction {@db.execute 'DROP TABLE test;'}
+    @db.transaction{@db.execute 'DROP TABLE test;'}
     @db.sql.should == ['BEGIN', 'DROP TABLE test;', 'COMMIT']
   end
   
+  specify "should support transaction isolation levels" do
+    @db.meta_def(:supports_transaction_isolation_levels?){true}
+    [:uncommitted, :committed, :repeatable, :serializable].each do |l|
+      @db.transaction(:isolation=>l){@db.run "DROP TABLE #{l}"}
+    end
+    @db.sql.should == ['BEGIN', 'SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED', 'DROP TABLE uncommitted', 'COMMIT',
+                       'BEGIN', 'SET TRANSACTION ISOLATION LEVEL READ COMMITTED', 'DROP TABLE committed', 'COMMIT',
+                       'BEGIN', 'SET TRANSACTION ISOLATION LEVEL REPEATABLE READ', 'DROP TABLE repeatable', 'COMMIT',
+                       'BEGIN', 'SET TRANSACTION ISOLATION LEVEL SERIALIZABLE', 'DROP TABLE serializable', 'COMMIT']
+  end
+
+  specify "should allow specifying a default transaction isolation level" do
+    @db.meta_def(:supports_transaction_isolation_levels?){true}
+    [:uncommitted, :committed, :repeatable, :serializable].each do |l|
+      @db.transaction_isolation_level = l
+      @db.transaction{@db.run "DROP TABLE #{l}"}
+    end
+    @db.sql.should == ['BEGIN', 'SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED', 'DROP TABLE uncommitted', 'COMMIT',
+                       'BEGIN', 'SET TRANSACTION ISOLATION LEVEL READ COMMITTED', 'DROP TABLE committed', 'COMMIT',
+                       'BEGIN', 'SET TRANSACTION ISOLATION LEVEL REPEATABLE READ', 'DROP TABLE repeatable', 'COMMIT',
+                       'BEGIN', 'SET TRANSACTION ISOLATION LEVEL SERIALIZABLE', 'DROP TABLE serializable', 'COMMIT']
+  end
+  
   specify "should handle returning inside of the block by committing" do
     def @db.ret_commit
       transaction do
@@ -655,13 +809,31 @@ context "Database#transaction" do
     t.join
     @db.transactions.should be_empty
   end
+
+  if (!defined?(RUBY_ENGINE) or RUBY_ENGINE == 'ruby' or RUBY_ENGINE == 'rbx') and RUBY_VERSION < '1.9'
+    specify "should handle Thread#kill for transactions inside threads" do
+      q = Queue.new
+      q1 = Queue.new
+      t = Thread.new do
+        @db.transaction do
+          @db.execute 'DROP TABLE test'
+          q1.push nil
+          q.pop
+          @db.execute 'DROP TABLE test2'
+        end
+      end
+      q1.pop
+      t.kill
+      t.join
+      @db.sql.should == ['BEGIN', 'DROP TABLE test', 'ROLLBACK']
+    end
+  end
 end
 
-context "Database#transaction with savepoints" do
+describe "Database#transaction with savepoints" do
   before do
-    @db = Dummy3Database.new
+    @db = Dummy3Database.new{Dummy3Database::DummyConnection.new(@db)}
     @db.meta_def(:supports_savepoints?){true}
-    @db.pool.connection_proc = proc {Dummy3Database::DummyConnection.new(@db)}
   end
   
   specify "should wrap the supplied block with BEGIN + COMMIT statements" do
@@ -757,7 +929,7 @@ context "Database#transaction with savepoints" do
   end
 end
 
-context "A Database adapter with a scheme" do
+describe "A Database adapter with a scheme" do
   before do
     class CCC < Sequel::Database
       if defined?(DISCONNECTS)
@@ -798,6 +970,7 @@ context "A Database adapter with a scheme" do
     x = nil
     y = nil
     z = nil
+    returnValue = 'anything'
 
     p = proc do |c|
       c.should be_a_kind_of(CCC)
@@ -806,15 +979,16 @@ context "A Database adapter with a scheme" do
       z = y
       y = x
       x = c
+      returnValue
     end
-    Sequel::Database.connect('ccc://localhost/db', &p).should == nil
+    Sequel::Database.connect('ccc://localhost/db', &p).should == returnValue
     CCC::DISCONNECTS.should == [x]
 
-    Sequel.connect('ccc://localhost/db', &p).should == nil
+    Sequel.connect('ccc://localhost/db', &p).should == returnValue
     CCC::DISCONNECTS.should == [y, x]
 
     Sequel.send(:def_adapter_method, :ccc)
-    Sequel.ccc('db', :host=>'localhost', &p).should == nil
+    Sequel.ccc('db', :host=>'localhost', &p).should == returnValue
     CCC::DISCONNECTS.should == [z, y, x]
   end
 
@@ -825,26 +999,27 @@ context "A Database adapter with a scheme" do
     proc {Sequel.ccc('abc', 'def')}.should raise_error(Sequel::Error)
     
     c = Sequel.ccc('mydb')
+    p = proc{c.opts.delete_if{|k,v| k == :disconnection_proc || k == :single_threaded}}
     c.should be_a_kind_of(CCC)
-    c.opts.should == {:adapter=>:ccc, :database => 'mydb'}
+    p.call.should == {:adapter=>:ccc, :database => 'mydb', :adapter_class=>CCC}
     
     c = Sequel.ccc('mydb', :host => 'localhost')
     c.should be_a_kind_of(CCC)
-    c.opts.should == {:adapter=>:ccc, :database => 'mydb', :host => 'localhost'}
+    p.call.should == {:adapter=>:ccc, :database => 'mydb', :host => 'localhost', :adapter_class=>CCC}
     
     c = Sequel.ccc
     c.should be_a_kind_of(CCC)
-    c.opts.should == {:adapter=>:ccc}
+    p.call.should == {:adapter=>:ccc, :adapter_class=>CCC}
     
     c = Sequel.ccc(:database => 'mydb', :host => 'localhost')
     c.should be_a_kind_of(CCC)
-    c.opts.should == {:adapter=>:ccc, :database => 'mydb', :host => 'localhost'}
+    p.call.should == {:adapter=>:ccc, :database => 'mydb', :host => 'localhost', :adapter_class=>CCC}
   end
   
   specify "should be accessible through Sequel.connect with options" do
     c = Sequel.connect(:adapter => :ccc, :database => 'mydb')
     c.should be_a_kind_of(CCC)
-    c.opts.should == {:adapter => :ccc, :database => 'mydb'}
+    c.opts[:adapter].should == :ccc
   end
 
   specify "should be accessible through Sequel.connect with URL parameters" do
@@ -876,16 +1051,30 @@ context "A Database adapter with a scheme" do
     c.opts[:database].should == 'd[b]'
     c.opts[:host].should == 'domain\\instance'
   end
+
+  specify "should test the connection if test parameter is truthy" do
+    proc{Sequel.connect 'ccc:///d%5bb%5d?test=t'}.should raise_error(Sequel::DatabaseConnectionError)
+    proc{Sequel.connect 'ccc:///d%5bb%5d?test=1'}.should raise_error(Sequel::DatabaseConnectionError)
+    proc{Sequel.connect 'ccc:///d%5bb%5d', :test=>true}.should raise_error(Sequel::DatabaseConnectionError)
+    proc{Sequel.connect 'ccc:///d%5bb%5d', :test=>'t'}.should raise_error(Sequel::DatabaseConnectionError)
+  end
+
+  specify "should not test the connection if test parameter is not truthy" do
+    proc{Sequel.connect 'ccc:///d%5bb%5d?test=f'}.should_not raise_error
+    proc{Sequel.connect 'ccc:///d%5bb%5d?test=0'}.should_not raise_error
+    proc{Sequel.connect 'ccc:///d%5bb%5d', :test=>false}.should_not raise_error
+    proc{Sequel.connect 'ccc:///d%5bb%5d', :test=>'f'}.should_not raise_error
+  end
 end
 
-context "Sequel::Database.connect" do
+describe "Sequel::Database.connect" do
   specify "should raise an Error if not given a String or Hash" do
     proc{Sequel::Database.connect(nil)}.should raise_error(Sequel::Error)
     proc{Sequel::Database.connect(Object.new)}.should raise_error(Sequel::Error)
   end
 end
 
-context "An unknown database scheme" do
+describe "An unknown database scheme" do
   specify "should raise an error in Sequel::Database.connect" do
     proc {Sequel::Database.connect('ddd://localhost/db')}.should raise_error(Sequel::AdapterNotFound)
   end
@@ -895,7 +1084,7 @@ context "An unknown database scheme" do
   end
 end
 
-context "A broken adapter (lib is there but the class is not)" do
+describe "A broken adapter (lib is there but the class is not)" do
   before do
     @fn = File.join(File.dirname(__FILE__), '../../lib/sequel/adapters/blah.rb')
     File.open(@fn,'a'){}
@@ -910,35 +1099,35 @@ context "A broken adapter (lib is there but the class is not)" do
   end
 end
 
-context "A single threaded database" do
+describe "A single threaded database" do
   after do
     Sequel::Database.single_threaded = false
   end
   
-  specify "should use a SingleThreadedPool instead of a ConnectionPool" do
-    db = Sequel::Database.new(:single_threaded => true)
-    db.pool.should be_a_kind_of(Sequel::SingleThreadedPool)
+  specify "should use a SingleConnectionPool instead of a ConnectionPool" do
+    db = Sequel::Database.new(:single_threaded => true){123}
+    db.pool.should be_a_kind_of(Sequel::SingleConnectionPool)
   end
   
   specify "should be constructable using :single_threaded => true option" do
-    db = Sequel::Database.new(:single_threaded => true)
-    db.pool.should be_a_kind_of(Sequel::SingleThreadedPool)
+    db = Sequel::Database.new(:single_threaded => true){123}
+    db.pool.should be_a_kind_of(Sequel::SingleConnectionPool)
   end
   
   specify "should be constructable using Database.single_threaded = true" do
     Sequel::Database.single_threaded = true
-    db = Sequel::Database.new
-    db.pool.should be_a_kind_of(Sequel::SingleThreadedPool)
+    db = Sequel::Database.new{123}
+    db.pool.should be_a_kind_of(Sequel::SingleConnectionPool)
   end
 
   specify "should be constructable using Sequel.single_threaded = true" do
     Sequel.single_threaded = true
-    db = Sequel::Database.new
-    db.pool.should be_a_kind_of(Sequel::SingleThreadedPool)
+    db = Sequel::Database.new{123}
+    db.pool.should be_a_kind_of(Sequel::SingleConnectionPool)
   end
 end
 
-context "A single threaded database" do
+describe "A single threaded database" do
   before do
     conn = 1234567
     @db = Sequel::Database.new(:single_threaded => true) do
@@ -951,16 +1140,28 @@ context "A single threaded database" do
     @db.pool.hold {|c| c.should == 1234568}
   end
   
-  specify "should convert an Exception into a RuntimeError" do
-    db = Sequel::Database.new(:single_threaded => true) do
-      raise Exception
-    end
-    
-    proc {db.pool.hold {|c|}}.should raise_error(RuntimeError)
+  specify "should disconnect correctly" do
+    def @db.disconnect_connection(c); @dc = c end
+    def @db.dc; @dc end
+    x = nil
+    @db.pool.hold{|c| x = c}
+    @db.pool.hold{|c| c.should == x}
+    proc{@db.disconnect}.should_not raise_error
+    @db.dc.should == x
+  end
+  
+  specify "should convert an Exception on connection into a DatabaseConnectionError" do
+    db = Sequel::Database.new(:single_threaded => true, :servers=>{}){raise Exception}
+    proc {db.pool.hold {|c|}}.should raise_error(Sequel::DatabaseConnectionError)
+  end
+  
+  specify "should raise a DatabaseConnectionError if the connection proc returns nil" do
+    db = Sequel::Database.new(:single_threaded => true, :servers=>{}){nil}
+    proc {db.pool.hold {|c|}}.should raise_error(Sequel::DatabaseConnectionError)
   end
 end
 
-context "A database" do
+describe "A database" do
   before do
     Sequel::Database.single_threaded = false
   end
@@ -970,7 +1171,7 @@ context "A database" do
   end
   
   specify "should have single_threaded? respond to true if in single threaded mode" do
-    db = Sequel::Database.new(:single_threaded => true)
+    db = Sequel::Database.new(:single_threaded => true){1234}
     db.should be_single_threaded
     
     db = Sequel::Database.new(:max_options => 1)
@@ -981,10 +1182,10 @@ context "A database" do
     
     Sequel::Database.single_threaded = true
     
-    db = Sequel::Database.new
+    db = Sequel::Database.new{123}
     db.should be_single_threaded
     
-    db = Sequel::Database.new(:max_options => 4)
+    db = Sequel::Database.new(:max_options => 4){123}
     db.should be_single_threaded
   end
   
@@ -1007,7 +1208,7 @@ context "A database" do
   end
 end
 
-context "Database#fetch" do
+describe "Database#fetch" do
   before do
     @db = Sequel::Database.new
     c = Class.new(Sequel::Dataset) do
@@ -1058,7 +1259,7 @@ context "Database#fetch" do
 end
 
 
-context "Database#[]" do
+describe "Database#[]" do
   before do
     @db = Sequel::Database.new
   end
@@ -1081,7 +1282,7 @@ context "Database#[]" do
   end
 end
 
-context "Database#create_view" do
+describe "Database#create_view" do
   before do
     @db = DummyDatabase.new
   end
@@ -1103,7 +1304,7 @@ context "Database#create_view" do
   end
 end
 
-context "Database#create_or_replace_view" do
+describe "Database#create_or_replace_view" do
   before do
     @db = DummyDatabase.new
   end
@@ -1125,7 +1326,7 @@ context "Database#create_or_replace_view" do
   end
 end
 
-context "Database#drop_view" do
+describe "Database#drop_view" do
   before do
     @db = DummyDatabase.new
   end
@@ -1139,7 +1340,7 @@ context "Database#drop_view" do
   end
 end
 
-context "Database#alter_table_sql" do
+describe "Database#alter_table_sql" do
   before do
     @db = DummyDatabase.new
   end
@@ -1149,7 +1350,7 @@ context "Database#alter_table_sql" do
   end
 end
 
-context "Database#inspect" do
+describe "Database#inspect" do
   before do
     @db = DummyDatabase.new
     
@@ -1161,7 +1362,7 @@ context "Database#inspect" do
   end
 end
 
-context "Database#get" do
+describe "Database#get" do
   before do
     @c = Class.new(DummyDatabase) do
       def dataset
@@ -1194,7 +1395,7 @@ context "Database#get" do
   end
 end
 
-context "Database#call" do
+describe "Database#call" do
   specify "should call the prepared statement with the given name" do
     db = MockDatabase.new
     db[:items].prepare(:select, :select_all)
@@ -1205,25 +1406,25 @@ context "Database#call" do
   end
 end
 
-context "Database#server_opts" do
+describe "Database#server_opts" do
   specify "should return the general opts if no :servers option is used" do
     opts = {:host=>1, :database=>2}
-    MockDatabase.new(opts).send(:server_opts, :server1).should == {:host=>1, :database=>2}
+    MockDatabase.new(opts).send(:server_opts, :server1)[:host].should == 1
   end
   
   specify "should return the general opts if entry for the server is present in the :servers option" do
     opts = {:host=>1, :database=>2, :servers=>{}}
-    MockDatabase.new(opts).send(:server_opts, :server1).should == {:host=>1, :database=>2}
+    MockDatabase.new(opts).send(:server_opts, :server1)[:host].should == 1
   end
   
   specify "should return the general opts merged with the specific opts if given as a hash" do
     opts = {:host=>1, :database=>2, :servers=>{:server1=>{:host=>3}}}
-    MockDatabase.new(opts).send(:server_opts, :server1).should == {:host=>3, :database=>2}
+    MockDatabase.new(opts).send(:server_opts, :server1)[:host].should == 3
   end
   
   specify "should return the sgeneral opts merged with the specific opts if given as a proc" do
     opts = {:host=>1, :database=>2, :servers=>{:server1=>proc{|db| {:host=>4}}}}
-    MockDatabase.new(opts).send(:server_opts, :server1).should == {:host=>4, :database=>2}
+    MockDatabase.new(opts).send(:server_opts, :server1)[:host].should == 4
   end
   
   specify "should raise an error if the specific opts is not a proc or hash" do
@@ -1232,7 +1433,167 @@ context "Database#server_opts" do
   end
 end
 
-context "Database#raise_error" do
+describe "Database#add_servers" do
+  before do
+    @db = MockDatabase.new(:host=>1, :database=>2, :servers=>{:server1=>{:host=>3}})
+    def @db.connect(server)
+      server_opts(server)
+    end
+    def @db.disconnect_connection(c)
+    end
+  end
+
+  specify "should add new servers to the connection pool" do
+    @db.synchronize{|c| c[:host].should == 1}
+    @db.synchronize(:server1){|c| c[:host].should == 3}
+    @db.synchronize(:server2){|c| c[:host].should == 1}
+
+    @db.add_servers(:server2=>{:host=>6})
+    @db.synchronize{|c| c[:host].should == 1}
+    @db.synchronize(:server1){|c| c[:host].should == 3}
+    @db.synchronize(:server2){|c| c[:host].should == 6}
+
+    @db.disconnect
+    @db.synchronize{|c| c[:host].should == 1}
+    @db.synchronize(:server1){|c| c[:host].should == 3}
+    @db.synchronize(:server2){|c| c[:host].should == 6}
+  end
+
+  specify "should replace options for future connections to existing servers" do
+    @db.synchronize{|c| c[:host].should == 1}
+    @db.synchronize(:server1){|c| c[:host].should == 3}
+    @db.synchronize(:server2){|c| c[:host].should == 1}
+
+    @db.add_servers(:default=>proc{{:host=>4}}, :server1=>{:host=>8})
+    @db.synchronize{|c| c[:host].should == 1}
+    @db.synchronize(:server1){|c| c[:host].should == 3}
+    @db.synchronize(:server2){|c| c[:host].should == 1}
+
+    @db.disconnect
+    @db.synchronize{|c| c[:host].should == 4}
+    @db.synchronize(:server1){|c| c[:host].should == 8}
+    @db.synchronize(:server2){|c| c[:host].should == 4}
+  end
+end
+
+describe "Database#remove_servers" do
+  before do
+    @db = MockDatabase.new(:host=>1, :database=>2, :servers=>{:server1=>{:host=>3}, :server2=>{:host=>4}})
+    def @db.connect(server)
+      server_opts(server)
+    end
+    def @db.disconnect_connection(c)
+    end
+  end
+
+  specify "should remove servers from the connection pool" do
+    @db.synchronize{|c| c[:host].should == 1}
+    @db.synchronize(:server1){|c| c[:host].should == 3}
+    @db.synchronize(:server2){|c| c[:host].should == 4}
+
+    @db.remove_servers(:server1, :server2)
+    @db.synchronize{|c| c[:host].should == 1}
+    @db.synchronize(:server1){|c| c[:host].should == 1}
+    @db.synchronize(:server2){|c| c[:host].should == 1}
+  end
+  
+  specify "should accept arrays of symbols" do
+    @db.remove_servers([:server1, :server2])
+    @db.synchronize{|c| c[:host].should == 1}
+    @db.synchronize(:server1){|c| c[:host].should == 1}
+    @db.synchronize(:server2){|c| c[:host].should == 1}
+  end
+
+  specify "should allow removal while connections are still open" do
+    @db.synchronize do |c1|
+      c1[:host].should == 1
+      @db.synchronize(:server1) do |c2|
+        c2[:host].should == 3
+        @db.synchronize(:server2) do |c3|
+          c3[:host].should == 4
+          @db.remove_servers(:server1, :server2)
+            @db.synchronize(:server1) do |c4|
+              c4.should_not == c2
+              c4.should == c1
+              c4[:host].should == 1
+              @db.synchronize(:server2) do |c5|
+                c5.should_not == c3
+                c5.should == c1
+                c5[:host].should == 1
+              end
+            end
+          c3[:host].should == 4
+         end
+        c2[:host].should == 3
+      end
+      c1[:host].should == 1
+    end
+  end
+end
+
+describe "Database#each_server with do/jdbc adapter connection string without :adapter option" do
+  before do
+    klass = Class.new(Sequel::Database)
+    klass.should_receive(:adapter_class).once.with(:jdbc).and_return(MockDatabase)
+    @db = klass.connect('jdbc:blah:', :host=>1, :database=>2, :servers=>{:server1=>{:host=>3}})
+    def @db.connect(server)
+      server_opts(server)
+    end
+    def @db.disconnect_connection(c)
+    end
+  end
+
+  specify "should yield a separate database object for each server" do
+    hosts = []
+    @db.each_server do |db|
+      db.should be_a_kind_of(Sequel::Database)
+      db.should_not == @db
+      db.opts[:adapter_class].should == MockDatabase
+      db.opts[:database].should == 2
+      hosts << db.opts[:host]
+    end
+    hosts.sort.should == [1, 3]
+  end
+end
+
+describe "Database#each_server" do
+  before do
+    @db = Sequel.connect(:adapter=>:mock, :host=>1, :database=>2, :servers=>{:server1=>{:host=>3}, :server2=>{:host=>4}})
+    def @db.connect(server)
+      server_opts(server)
+    end
+    def @db.disconnect_connection(c)
+    end
+  end
+
+  specify "should yield a separate database object for each server" do
+    hosts = []
+    @db.each_server do |db|
+      db.should be_a_kind_of(Sequel::Database)
+      db.should_not == @db
+      db.opts[:adapter].should == :mock
+      db.opts[:database].should == 2
+      hosts << db.opts[:host]
+    end
+    hosts.sort.should == [1, 3, 4]
+  end
+
+  specify "should disconnect and remove entry from Sequel::DATABASES after use" do
+    dbs = []
+    dcs = []
+    @db.each_server do |db|
+      dbs << db
+      Sequel::DATABASES.should include(db)
+      db.meta_def(:disconnect){dcs << db}
+    end
+    dbs.each do |db|
+      Sequel::DATABASES.should_not include(db)
+    end
+    dbs.should == dcs
+  end
+end
+  
+describe "Database#raise_error" do
   specify "should reraise if the exception class is not in opts[:classes]" do
     e = Class.new(StandardError)
     proc{MockDatabase.new.send(:raise_error, e.new(''), :classes=>[])}.should raise_error(e)
@@ -1251,7 +1612,7 @@ context "Database#raise_error" do
   end
 end
 
-context "Database#typecast_value" do
+describe "Database#typecast_value" do
   before do
     @db = Sequel::Database.new
   end
@@ -1285,7 +1646,7 @@ context "Database#typecast_value" do
   end
 end
 
-context "Database#blank_object?" do
+describe "Database#blank_object?" do
   specify "should return whether the object is considered blank" do
     db = Sequel::Database.new
     c = lambda{|meth, value| Class.new{define_method(meth){value}}.new}
@@ -1310,7 +1671,7 @@ context "Database#blank_object?" do
   end
 end
 
-context "Database#schema_autoincrementing_primary_key?" do
+describe "Database#schema_autoincrementing_primary_key?" do
   specify "should whether the parsed schema row indicates a primary key" do
     m = Sequel::Database.new.method(:schema_autoincrementing_primary_key?)
     m.call(:primary_key=>true).should == true
@@ -1318,13 +1679,25 @@ context "Database#schema_autoincrementing_primary_key?" do
   end
 end
 
-context "Database#supports_savepoints?" do
+describe "Database#supports_savepoints?" do
   specify "should be false by default" do
     Sequel::Database.new.supports_savepoints?.should == false
   end
 end
 
-context "Database#input_identifier_meth" do
+describe "Database#supports_prepared_transactions?" do
+  specify "should be false by default" do
+    Sequel::Database.new.supports_prepared_transactions?.should == false
+  end
+end
+
+describe "Database#supports_transaction_isolation_levels?" do
+  specify "should be false by default" do
+    Sequel::Database.new.supports_transaction_isolation_levels?.should == false
+  end
+end
+
+describe "Database#input_identifier_meth" do
   specify "should be the input_identifer method of a default dataset for this database" do
     db = Sequel::Database.new
     db.send(:input_identifier_meth).call(:a).should == 'a'
@@ -1333,7 +1706,7 @@ context "Database#input_identifier_meth" do
   end
 end
 
-context "Database#output_identifier_meth" do
+describe "Database#output_identifier_meth" do
   specify "should be the output_identifer method of a default dataset for this database" do
     db = Sequel::Database.new
     db.send(:output_identifier_meth).call('A').should == :A
@@ -1342,7 +1715,7 @@ context "Database#output_identifier_meth" do
   end
 end
 
-context "Database#metadata_dataset" do
+describe "Database#metadata_dataset" do
   specify "should be a dataset with the default settings for identifier_input_method and identifier_output_method" do
     ds = Sequel::Database.new.send(:metadata_dataset)
     ds.literal(:a).should == 'A'
@@ -1350,7 +1723,7 @@ context "Database#metadata_dataset" do
   end
 end
 
-context "Database#column_schema_to_ruby_default" do
+describe "Database#column_schema_to_ruby_default" do
   specify "should handle converting many default formats" do
     db = Sequel::Database.new
     m = db.method(:column_schema_to_ruby_default)
