@@ -1,9 +1,9 @@
-require File.join(File.dirname(__FILE__), 'spec_helper')
+require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper')
 
 Regexp.send(:include, Sequel::SQL::StringMethods)
 String.send(:include, Sequel::SQL::StringMethods)
 
-context "Blockless Ruby Filters" do
+describe "Blockless Ruby Filters" do
   before do
     db = Sequel::Database.new
     db.quote_identifiers = false
@@ -216,7 +216,7 @@ context "Blockless Ruby Filters" do
   it "should support AND conditions via &" do
     @d.l(:x & :y).should == '(x AND y)'
     @d.l(:x.sql_boolean & :y).should == '(x AND y)'
-    @d.l(:x & :y & :z).should == '((x AND y) AND z)'
+    @d.l(:x & :y & :z).should == '(x AND y AND z)'
     @d.l(:x & {:y => :z}).should == '(x AND (y = z))'
     @d.l({:y => :z} & :x).should == '((y = z) AND x)'
     @d.l({:x => :a} & {:y => :z}).should == '((x = a) AND (y = z))'
@@ -229,7 +229,7 @@ context "Blockless Ruby Filters" do
   it "should support OR conditions via |" do
     @d.l(:x | :y).should == '(x OR y)'
     @d.l(:x.sql_boolean | :y).should == '(x OR y)'
-    @d.l(:x | :y | :z).should == '((x OR y) OR z)'
+    @d.l(:x | :y | :z).should == '(x OR y OR z)'
     @d.l(:x | {:y => :z}).should == '(x OR (y = z))'
     @d.l({:y => :z} | :x).should == '((y = z) OR x)'
     @d.l({:x => :a} | {:y => :z}).should == '((x = a) OR (y = z))'
@@ -263,7 +263,7 @@ context "Blockless Ruby Filters" do
     @d.l('x'.lit + 1 > 100).should == '((x + 1) > 100)'
     @d.l(('x'.lit * :y) < 100.01).should == '((x * y) < 100.01)'
     @d.l(('x'.lit - :y/2) >= 100000000000000000000000000000000000).should == '((x - (y / 2)) >= 100000000000000000000000000000000000)'
-    @d.l(('z'.lit * (('x'.lit / :y)/(:x + :y))) <= 100).should == '((z * ((x / y) / (x + y))) <= 100)'
+    @d.l(('z'.lit * (('x'.lit / :y)/(:x + :y))) <= 100).should == '((z * (x / y / (x + y))) <= 100)'
     @d.l(~(((('x'.lit - :y)/(:x + :y))*:z) <= 100)).should == '((((x - y) / (x + y)) * z) > 100)'
   end
 
@@ -350,13 +350,13 @@ context "Blockless Ruby Filters" do
     @d.lit([:x, 1, :y].sql_string_join(:y__z)).should == "(x || y.z || '1' || y.z || y)"
     @d.lit([:x, 1, :y].sql_string_join(1)).should == "(x || '1' || '1' || '1' || y)"
     @d.lit([:x, :y].sql_string_join('y.x || x.y'.lit)).should == "(x || y.x || x.y || y)"
-    @d.lit([[:x, :y].sql_string_join, [:a, :b].sql_string_join].sql_string_join).should == "((x || y) || (a || b))"
+    @d.lit([[:x, :y].sql_string_join, [:a, :b].sql_string_join].sql_string_join).should == "(x || y || a || b)"
   end
 
   it "should support StringExpression#+ for concatenation of SQL strings" do
     @d.lit(:x.sql_string + :y).should == '(x || y)'
-    @d.lit([:x].sql_string_join + :y).should == '((x) || y)'
-    @d.lit([:x, :z].sql_string_join(' ') + :y).should == "((x || ' ' || z) || y)"
+    @d.lit([:x].sql_string_join + :y).should == '(x || y)'
+    @d.lit([:x, :z].sql_string_join(' ') + :y).should == "(x || ' ' || z || y)"
   end
 
   it "should be supported inside blocks" do
@@ -399,6 +399,44 @@ context "Blockless Ruby Filters" do
     y.lit.should == y
   end
 
+  it "should return have .sql_literal operate like .to_s" do
+    y = :x + 1
+    y.sql_literal(@d).should == '(x + 1)'
+    y.sql_literal(@d).should == y.to_s(@d)
+    y.sql_literal(@d).should == @d.literal(y)
+  end
+
+  it "should support SQL::Constants" do
+    @d.l({:x => Sequel::NULL}).should == '(x IS NULL)'
+    @d.l({:x => Sequel::NOTNULL}).should == '(x IS NOT NULL)'
+    @d.l({:x => Sequel::TRUE}).should == '(x IS TRUE)'
+    @d.l({:x => Sequel::FALSE}).should == '(x IS FALSE)'
+    @d.l({:x => Sequel::SQLTRUE}).should == '(x IS TRUE)'
+    @d.l({:x => Sequel::SQLFALSE}).should == '(x IS FALSE)'
+  end
+  
+  it "should support negation of SQL::Constants" do
+    @d.l(~{:x => Sequel::NULL}).should == '(x IS NOT NULL)'
+    @d.l(~{:x => Sequel::NOTNULL}).should == '(x IS NULL)'
+    @d.l(~{:x => Sequel::TRUE}).should == '(x IS NOT TRUE)'
+    @d.l(~{:x => Sequel::FALSE}).should == '(x IS NOT FALSE)'
+    @d.l(~{:x => Sequel::SQLTRUE}).should == '(x IS NOT TRUE)'
+    @d.l(~{:x => Sequel::SQLFALSE}).should == '(x IS NOT FALSE)'
+  end
+  
+  it "should support direct negation of SQL::Constants" do
+    @d.l({:x => ~Sequel::NULL}).should == '(x IS NOT NULL)'
+    @d.l({:x => ~Sequel::NOTNULL}).should == '(x IS NULL)'
+    @d.l({:x => ~Sequel::TRUE}).should == '(x IS FALSE)'
+    @d.l({:x => ~Sequel::FALSE}).should == '(x IS TRUE)'
+    @d.l({:x => ~Sequel::SQLTRUE}).should == '(x IS FALSE)'
+    @d.l({:x => ~Sequel::SQLFALSE}).should == '(x IS TRUE)'
+  end
+  
+  it "should raise an error if trying to invert an invalid SQL::Constant" do
+    proc{~Sequel::CURRENT_DATE}.should raise_error(Sequel::Error)
+  end
+
   it "should raise an error if trying to create an invalid complex expression" do
     proc{Sequel::SQL::ComplexExpression.new(:BANG, 1, 2)}.should raise_error(Sequel::Error)
   end
@@ -456,7 +494,7 @@ context "Blockless Ruby Filters" do
   end
 end
 
-context Sequel::SQL::VirtualRow do
+describe Sequel::SQL::VirtualRow do
   before do
     db = Sequel::Database.new
     db.quote_identifiers = true
@@ -531,6 +569,10 @@ context Sequel::SQL::VirtualRow do
     @d.l{rank(:over, :frame=>:rows){}}.should == 'rank() OVER (ROWS UNBOUNDED PRECEDING)'
   end
 
+  it "should support :frame=>'some string' option for window function calls" do
+    @d.l{rank(:over, :frame=>'RANGE BETWEEN 3 PRECEDING AND CURRENT ROW'){}}.should == 'rank() OVER (RANGE BETWEEN 3 PRECEDING AND CURRENT ROW)'
+  end
+
   it "should raise an error if an invalid :frame option is used" do
     proc{@d.l{rank(:over, :frame=>:blah){}}}.should raise_error(Sequel::Error)
   end
@@ -550,4 +592,21 @@ context Sequel::SQL::VirtualRow do
     @d.l{date < Sequel::CURRENT_DATE}.should == "(\"date\" < CURRENT_DATE)"
     @d.l{num < Math::PI.to_i}.should == "(\"num\" < 3)"
   end
+  
+  it "should deal with methods added to Object after requiring Sequel" do
+    class Object
+      def adsoiwemlsdaf; 42; end
+    end
+    Sequel::BasicObject.remove_methods!
+    @d.l{a > adsoiwemlsdaf}.should == '("a" > "adsoiwemlsdaf")'
+  end
+  
+  it "should deal with private methods added to Kernel after requiring Sequel" do
+    module Kernel
+      private
+      def adsoiwemlsdaf2; 42; end
+    end
+    Sequel::BasicObject.remove_methods!
+    @d.l{a > adsoiwemlsdaf2}.should == '("a" > "adsoiwemlsdaf2")'
+  end
 end
