@@ -1,4 +1,4 @@
-require File.join(File.dirname(__FILE__), "spec_helper")
+require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
 
 describe Sequel::Model, "caching" do
   before do
@@ -57,10 +57,12 @@ describe Sequel::Model, "caching" do
       def update(values)
         $sqls << update_sql(values)
         $cache_dataset_row.merge!(values)
+        1
       end
       
       def delete
         $sqls << delete_sql
+        1
       end
     })
     
