@@ -1,4 +1,4 @@
-require File.join(File.dirname(__FILE__), "spec_helper")
+require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
 
 describe Sequel::Model do
   before do
@@ -9,11 +9,30 @@ describe Sequel::Model do
     end
   end
   
-  specify "should respond to validates, validations, has_validations?" do
+  specify "should respond to validations, has_validations?, and validation_reflections" do
     @c.should respond_to(:validations)
     @c.should respond_to(:has_validations?)
+    @c.should respond_to(:validation_reflections)
   end
   
+  specify "should be able to reflect on validations" do
+    @c.validation_reflections.should == {}
+    @c.validates_acceptance_of(:a)
+    @c.validation_reflections.should == {:a=>[[:acceptance, {:tag=>:acceptance, :message=>"is not accepted", :allow_nil=>true, :accept=>"1"}]]}
+    @c.validates_presence_of(:a)
+    @c.validation_reflections[:a].length.should == 2
+    @c.validation_reflections[:a].last.should == [:presence, {:tag=>:presence, :message=>"is not present"}]
+  end
+
+  specify "should handle validation reflections correctly when subclassing" do
+    @c.validates_acceptance_of(:a)
+    c = Class.new(@c)
+    c.validation_reflections.map{|k,v| k}.should == [:a]
+    c.validates_presence_of(:a)
+    @c.validation_reflections.should == {:a=>[[:acceptance, {:tag=>:acceptance, :message=>"is not accepted", :allow_nil=>true, :accept=>"1"}]]}
+    c.validation_reflections[:a].last.should == [:presence, {:tag=>:presence, :message=>"is not present"}]
+  end
+
   specify "should acccept validation definitions using validates_each" do
     @c.validates_each(:xx, :yy) {|o, a, v| o.errors[a] << 'too low' if v < 50}
     o = @c.new
@@ -322,6 +341,20 @@ describe Sequel::Model do
     @m.should be_valid
     @m.value = '123456'
     @m.should_not be_valid
+    @m.errors[:value].should == ['is too long']
+    @m.value = nil
+    @m.should_not be_valid
+    @m.errors[:value].should == ['is not present']
+  end
+
+  specify "should validate length_of with maximum using customized error messages" do
+    @c.validates_length_of :value, :maximum => 5, :too_long=>'tl', :nil_message=>'np'
+    @m.value = '123456'
+    @m.should_not be_valid
+    @m.errors[:value].should == ['tl']
+    @m.value = nil
+    @m.should_not be_valid
+    @m.errors[:value].should == ['np']
   end
 
   specify "should validate length_of with minimum" do
@@ -550,7 +583,7 @@ describe Sequel::Model do
   end
 end
 
-context "Superclass validations" do
+describe "Superclass validations" do
   before do
     @c1 = Class.new(Sequel::Model)
     @c1.class_eval do
@@ -602,7 +635,7 @@ context "Superclass validations" do
   end
 end
 
-context ".validates with block" do
+describe ".validates with block" do
   specify "should support calling .each" do
     @c = Class.new(Sequel::Model)
     @c.class_eval do
@@ -944,10 +977,10 @@ describe "Model#save" do
   before do
     @c = Class.new(Sequel::Model(:people))
     @c.class_eval do
-      columns :id
+      columns :id, :x
 
-      validates_each :id do |o, a, v|
-        o.errors[a] << 'blah' unless v == 5
+      validates_each :x do |o, a, v|
+        o.errors[a] << 'blah' unless v == 7
       end
     end
     @m = @c.load(:id => 4, :x=>6)
@@ -960,10 +993,10 @@ describe "Model#save" do
     @m.save
     MODEL_DB.sqls.should be_empty
     
-    @m.id = 5
+    @m.x = 7
     @m.should be_valid
     @m.save.should_not be_false
-    MODEL_DB.sqls.should == ['UPDATE people SET x = 6 WHERE (id = 5)']
+    MODEL_DB.sqls.should == ['UPDATE people SET x = 7 WHERE (id = 4)']
   end
   
   specify "should skip validations if the :validate=>false option is used" do
