@@ -52,6 +52,7 @@ describe "Model#save" do
 
   it "should use dataset's insert_select method if present" do
     ds = @c.dataset = @c.dataset.clone
+    def ds.supports_insert_select?() true end
     def ds.insert_select(hash)
       execute("INSERT INTO items (y) VALUES (2)")
       {:y=>2}
@@ -63,6 +64,12 @@ describe "Model#save" do
     MODEL_DB.sqls.should == ["INSERT INTO items (y) VALUES (2)"]
   end
 
+  it "should not use dataset's insert_select method if specific columns are selected" do
+    ds = @c.dataset = @c.dataset.select(:y)
+    ds.should_not_receive(:insert_select)
+    @c.new(:x => 1).save
+  end
+
   it "should use value returned by insert as the primary key and refresh the object" do
     @c.dataset.meta_def(:insert){|h| super(h); 13}
     o = @c.new(:x => 11)
@@ -1161,6 +1168,29 @@ describe Sequel::Model, "#initialize" do
     m.values.should == {:x => 2}
   end
 end
+  
+describe Sequel::Model, "#initialize_set" do
+  before do
+    @c = Class.new(Sequel::Model){columns :id, :x, :y}
+  end
+
+  specify "should be called by initialize to set the column values" do
+    @c.send(:define_method, :initialize_set){|h| set(:y => 3)}
+    @c.new(:x => 2).values.should == {:y => 3}
+  end
+
+  specify "should be called with the hash given to initialize " do
+    x = nil
+    @c.send(:define_method, :initialize_set){|y| x = y}
+    @c.new(:x => 2)
+    x.should == {:x => 2}
+  end
+
+  specify "should not cause columns modified by the method to be considered as changed" do
+    @c.send(:define_method, :initialize_set){|h| set(:y => 3)}
+    @c.new(:x => 2).changed_columns.should == []
+  end
+end
 
 describe Sequel::Model, ".create" do
 
