@@ -1,13 +1,18 @@
 require 'rubygems'
 unless Object.const_defined?('Sequel')
-  $:.unshift(File.join(File.dirname(__FILE__), "../../lib/"))
+  $:.unshift(File.join(File.dirname(File.expand_path(__FILE__)), "../../lib/"))
   require 'sequel/core'
 end
 unless Sequel.const_defined?('Model')
-  $:.unshift(File.join(File.dirname(__FILE__), "../../lib/"))
+  $:.unshift(File.join(File.dirname(File.expand_path(__FILE__)), "../../lib/"))
   require 'sequel/model'
 end
 
+if ENV['SEQUEL_COLUMNS_INTROSPECTION']
+  Sequel.extension :columns_introspection
+  Sequel::Dataset.introspect_all_columns
+end
+
 class MockDataset < Sequel::Dataset
   def insert(*args)
     @db.execute insert_sql(*args)
@@ -15,10 +20,12 @@ class MockDataset < Sequel::Dataset
   
   def update(*args)
     @db.execute update_sql(*args)
+    1
   end
   
   def delete(*args)
     @db.execute delete_sql(*args)
+    1
   end
   
   def fetch_rows(sql)
